@@ -36,7 +36,6 @@
 #include <stdlib.h>
 #include <gtk/gtk.h>
 #include <vte/vte.h>
-#include <gconf/gconf-client.h>
 #include <canberra-gtk.h>
 
 #include "gcm-calibrate-argyll.h"
@@ -44,21 +43,17 @@
 #include "gcm-utils.h"
 #include "gcm-screen.h"
 #include "gcm-print.h"
+#include "gcm-xyz.h"
 #include "gcm-calibrate-dialog.h"
 
 #include "egg-debug.h"
 
+//#define FIXED_ARGYLL
+
 static void     gcm_calibrate_argyll_finalize	(GObject     *object);
 
 #define GCM_CALIBRATE_ARGYLL_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GCM_TYPE_CALIBRATE_ARGYLL, GcmCalibrateArgyllPrivate))
 
-typedef enum {
-	GCM_CALIBRATE_ARGYLL_PRECISION_SHORT,
-	GCM_CALIBRATE_ARGYLL_PRECISION_NORMAL,
-	GCM_CALIBRATE_ARGYLL_PRECISION_LONG,
-	GCM_CALIBRATE_ARGYLL_PRECISION_LAST
-} GcmCalibrateArgyllPrecision;
-
 typedef enum {
 	GCM_CALIBRATE_ARGYLL_STATE_IDLE,
 	GCM_CALIBRATE_ARGYLL_STATE_WAITING_FOR_STDIN,
@@ -75,8 +70,6 @@ typedef enum {
 struct _GcmCalibrateArgyllPrivate
 {
 	guint				 display;
-	GConfClient			*gconf_client;
-	GcmCalibrateArgyllPrecision	 precision;
 	GMainLoop			*loop;
 	GtkWidget			*terminal;
 	GcmCalibrateDialog		*calibrate_dialog;
@@ -86,9 +79,11 @@ struct _GcmCalibrateArgyllPrivate
 	glong				 vte_previous_row;
 	glong				 vte_previous_col;
 	gboolean			 already_on_window;
+	gboolean			 done_calibrate;
 	GcmCalibrateArgyllState		 state;
 	GcmPrint			*print;
 	const gchar			*argyllcms_ok;
+	gboolean			 done_spot_read;
 };
 
 enum {
@@ -98,34 +93,19 @@ enum {
 
 G_DEFINE_TYPE (GcmCalibrateArgyll, gcm_calibrate_argyll, GCM_TYPE_CALIBRATE)
 
-/**
- * gcm_calibrate_argyll_precision_from_string:
- **/
-static GcmCalibrateArgyllPrecision
-gcm_calibrate_argyll_precision_from_string (const gchar *string)
-{
-	if (g_strcmp0 (string, "short") == 0)
-		return GCM_CALIBRATE_ARGYLL_PRECISION_SHORT;
-	if (g_strcmp0 (string, "normal") == 0)
-		return GCM_CALIBRATE_ARGYLL_PRECISION_NORMAL;
-	if (g_strcmp0 (string, "long") == 0)
-		return GCM_CALIBRATE_ARGYLL_PRECISION_LONG;
-	egg_warning ("failed to convert to precision: %s", string);
-	return GCM_CALIBRATE_ARGYLL_PRECISION_NORMAL;
-}
-
 /**
  * gcm_calibrate_argyll_get_quality_arg:
  **/
 static const gchar *
 gcm_calibrate_argyll_get_quality_arg (GcmCalibrateArgyll *calibrate_argyll)
 {
-	GcmCalibrateArgyllPrivate *priv = calibrate_argyll->priv;
 	GcmCalibrateReferenceKind reference_kind;
+	GcmCalibratePrecision precision;
 
 	/* get kind */
 	g_object_get (calibrate_argyll,
 		      "reference-kind", &reference_kind,
+		      "precision", &precision,
 		      NULL);
 
 	/* these have such low patch count, we only can do low quality */
@@ -134,11 +114,11 @@ gcm_calibrate_argyll_get_quality_arg (GcmCalibrateArgyll *calibrate_argyll)
 		return "-ql";
 
 	/* get the default precision */
-	if (priv->precision == GCM_CALIBRATE_ARGYLL_PRECISION_SHORT)
+	if (precision == GCM_CALIBRATE_PRECISION_SHORT)
 		return "-ql";
-	if (priv->precision == GCM_CALIBRATE_ARGYLL_PRECISION_NORMAL)
+	if (precision == GCM_CALIBRATE_PRECISION_NORMAL)
 		return "-qm";
-	if (priv->precision == GCM_CALIBRATE_ARGYLL_PRECISION_LONG)
+	if (precision == GCM_CALIBRATE_PRECISION_LONG)
 		return "-qh";
 	return "-qm";
 }
@@ -150,13 +130,18 @@ static guint
 gcm_calibrate_argyll_display_get_patches (GcmCalibrateArgyll *calibrate_argyll)
 {
 	guint patches = 250;
-	GcmCalibrateArgyllPrivate *priv = calibrate_argyll->priv;
+	GcmCalibratePrecision precision;
 
-	if (priv->precision == GCM_CALIBRATE_ARGYLL_PRECISION_SHORT)
+	/* get kind */
+	g_object_get (calibrate_argyll,
+		      "precision", &precision,
+		      NULL);
+
+	if (precision == GCM_CALIBRATE_PRECISION_SHORT)
 		patches = 100;
-	else if (priv->precision == GCM_CALIBRATE_ARGYLL_PRECISION_NORMAL)
+	else if (precision == GCM_CALIBRATE_PRECISION_NORMAL)
 		patches = 250;
-	else if (priv->precision == GCM_CALIBRATE_ARGYLL_PRECISION_LONG)
+	else if (precision == GCM_CALIBRATE_PRECISION_LONG)
 		patches = 500;
 	return patches;
 }
@@ -169,16 +154,19 @@ gcm_calibrate_argyll_printer_get_patches (GcmCalibrateArgyll *calibrate_argyll)
 {
 	guint patches = 180;
 	GcmColorimeterKind colorimeter_kind;
-	GcmCalibrateArgyllPrivate *priv = calibrate_argyll->priv;
+	GcmCalibratePrecision precision;
 
-	/* we care about the type */
-	g_object_get (calibrate_argyll, "colorimeter-kind", &colorimeter_kind, NULL);
+	/* we care about the kind */
+	g_object_get (calibrate_argyll,
+		      "colorimeter-kind", &colorimeter_kind,
+		      "precision", &precision,
+		      NULL);
 
-	if (priv->precision == GCM_CALIBRATE_ARGYLL_PRECISION_SHORT)
+	if (precision == GCM_CALIBRATE_PRECISION_SHORT)
 		patches = 90;
-	else if (priv->precision == GCM_CALIBRATE_ARGYLL_PRECISION_NORMAL)
+	else if (precision == GCM_CALIBRATE_PRECISION_NORMAL)
 		patches = 180;
-	else if (priv->precision == GCM_CALIBRATE_ARGYLL_PRECISION_LONG)
+	else if (precision == GCM_CALIBRATE_PRECISION_LONG)
 		patches = 360;
 
 	/* using double density, so we can double the patch count */
@@ -205,6 +193,10 @@ gcm_calibrate_argyll_get_colorimeter_image_attach (GcmCalibrateArgyll *calibrate
 		return "munki-attach.svg";
 	if (colorimeter_kind == GCM_COLORIMETER_KIND_SPYDER)
 		return "spyder-attach.svg";
+	if (colorimeter_kind == GCM_COLORIMETER_KIND_COLORIMTRE_HCFR)
+		return "hcfr-attach.svg";
+	if (colorimeter_kind == GCM_COLORIMETER_KIND_I1_PRO)
+		return "i1-attach.svg";
 	return NULL;
 }
 
@@ -289,15 +281,15 @@ out:
 }
 
 /**
- * gcm_calibrate_argyll_get_display_type:
+ * gcm_calibrate_argyll_get_display_kind:
  **/
 static gchar
-gcm_calibrate_argyll_get_display_type (GcmCalibrateArgyll *calibrate_argyll)
+gcm_calibrate_argyll_get_display_kind (GcmCalibrateArgyll *calibrate_argyll)
 {
 	GcmCalibrateDeviceKind device_kind;
 
 	g_object_get (calibrate_argyll,
-		      "device-kind", &device_kind,
+		      "calibrate-device-kind", &device_kind,
 		      NULL);
 
 	if (device_kind == GCM_CALIBRATE_DEVICE_KIND_LCD)
@@ -369,7 +361,7 @@ gcm_calibrate_argyll_display_neutralise (GcmCalibrateArgyll *calibrate_argyll, G
 {
 	gboolean ret = TRUE;
 	GcmCalibrateArgyllPrivate *priv = calibrate_argyll->priv;
-	gchar type;
+	gchar kind;
 	gchar *command = NULL;
 	gchar **argv = NULL;
 	GnomeRROutput *output;
@@ -409,7 +401,7 @@ gcm_calibrate_argyll_display_neutralise (GcmCalibrateArgyll *calibrate_argyll, G
 	}
 
 	/* get l-cd or c-rt */
-	type = gcm_calibrate_argyll_get_display_type (calibrate_argyll);
+	kind = gcm_calibrate_argyll_get_display_kind (calibrate_argyll);
 
 	/* TRANSLATORS: title, default paramters needed to calibrate_argyll */
 	title = _("Getting default parameters");
@@ -426,11 +418,14 @@ gcm_calibrate_argyll_display_neutralise (GcmCalibrateArgyll *calibrate_argyll, G
 	array = g_ptr_array_new_with_free_func (g_free);
 
 	/* setup the command */
-	g_ptr_array_add (array, g_strdup ("-v9"));
+#ifdef FIXED_ARGYLL
+	g_ptr_array_add (array, g_strdup (command));
+#endif
+	g_ptr_array_add (array, g_strdup ("-v"));
 	g_ptr_array_add (array, g_strdup ("-ql"));
 	g_ptr_array_add (array, g_strdup ("-m"));
 	g_ptr_array_add (array, g_strdup_printf ("-d%i", priv->display));
-	g_ptr_array_add (array, g_strdup_printf ("-y%c", type));
+	g_ptr_array_add (array, g_strdup_printf ("-y%c", kind));
 //	g_ptr_array_add (array, g_strdup ("-p 0.8,0.5,1.0"));
 	g_ptr_array_add (array, g_strdup (basename));
 	argv = gcm_utils_ptr_array_to_strv (array);
@@ -508,7 +503,7 @@ gcm_calibrate_argyll_display_read_chart (GcmCalibrateArgyll *calibrate_argyll, G
 	/* TRANSLATORS: title, patches are specific colours used in calibration */
 	title = _("Reading the patches");
 	/* TRANSLATORS: dialog message */
-	message = _("Reading the patches using the hardware device.");
+	message = _("Reading the patches using the color measuring instrument.");
 
 	/* push new messages into the UI */
 	gcm_calibrate_dialog_show (priv->calibrate_dialog, GCM_CALIBRATE_DIALOG_TAB_GENERIC, title, message);
@@ -519,7 +514,12 @@ gcm_calibrate_argyll_display_read_chart (GcmCalibrateArgyll *calibrate_argyll, G
 	array = g_ptr_array_new_with_free_func (g_free);
 
 	/* setup the command */
-	g_ptr_array_add (array, g_strdup ("-v9"));
+#ifdef FIXED_ARGYLL
+	g_ptr_array_add (array, g_strdup (command));
+#endif
+	g_ptr_array_add (array, g_strdup ("-v"));
+	if (priv->done_calibrate)
+		g_ptr_array_add (array, g_strdup ("-N"));
 	g_ptr_array_add (array, g_strdup (basename));
 	argv = gcm_utils_ptr_array_to_strv (array);
 	gcm_calibrate_argyll_debug_argv (command, argv);
@@ -577,13 +577,13 @@ gcm_calibrate_argyll_display_generate_patches (GcmCalibrateArgyll *calibrate_arg
 	gchar *working_path = NULL;
 	const gchar *title;
 	const gchar *message;
-	GcmDeviceTypeEnum device_type;
+	GcmDeviceKind device_kind;
 
 	/* get shared data */
 	g_object_get (calibrate_argyll,
 		      "basename", &basename,
 		      "working-path", &working_path,
-		      "device-type", &device_type,
+		      "device-kind", &device_kind,
 		      NULL);
 
 	/* get correct name of the command */
@@ -596,7 +596,7 @@ gcm_calibrate_argyll_display_generate_patches (GcmCalibrateArgyll *calibrate_arg
 	/* TRANSLATORS: title, patches are specific colours used in calibration */
 	title = _("Generating the patches");
 	/* TRANSLATORS: dialog message */
-	message = _("Generating the patches that will be measured with the hardware device.");
+	message = _("Generating the patches that will be measured with the color instrument.");
 
 	/* push new messages into the UI */
 	gcm_calibrate_dialog_show (priv->calibrate_dialog, GCM_CALIBRATE_DIALOG_TAB_GENERIC, title, message);
@@ -607,8 +607,11 @@ gcm_calibrate_argyll_display_generate_patches (GcmCalibrateArgyll *calibrate_arg
 	array = g_ptr_array_new_with_free_func (g_free);
 
 	/* setup the command */
-	g_ptr_array_add (array, g_strdup ("-v9"));
-	if (device_type == GCM_DEVICE_TYPE_ENUM_PRINTER) {
+#ifdef FIXED_ARGYLL
+	g_ptr_array_add (array, g_strdup (command));
+#endif
+	g_ptr_array_add (array, g_strdup ("-v"));
+	if (device_kind == GCM_DEVICE_KIND_PRINTER) {
 		/* print RGB */
 		g_ptr_array_add (array, g_strdup ("-d2"));
 
@@ -620,9 +623,9 @@ gcm_calibrate_argyll_display_generate_patches (GcmCalibrateArgyll *calibrate_arg
 	}
 
 	/* get number of patches */
-	if (device_type == GCM_DEVICE_TYPE_ENUM_DISPLAY)
+	if (device_kind == GCM_DEVICE_KIND_DISPLAY)
 		g_ptr_array_add (array, g_strdup_printf ("-f%i", gcm_calibrate_argyll_display_get_patches (calibrate_argyll)));
-	else if (device_type == GCM_DEVICE_TYPE_ENUM_PRINTER)
+	else if (device_kind == GCM_DEVICE_KIND_PRINTER)
 		g_ptr_array_add (array, g_strdup_printf ("-f%i", gcm_calibrate_argyll_printer_get_patches (calibrate_argyll)));
 
 	g_ptr_array_add (array, g_strdup (basename));
@@ -675,7 +678,7 @@ gcm_calibrate_argyll_display_draw_and_measure (GcmCalibrateArgyll *calibrate_arg
 {
 	gboolean ret = TRUE;
 	GcmCalibrateArgyllPrivate *priv = calibrate_argyll->priv;
-	gchar type;
+	gchar kind;
 	gchar *command = NULL;
 	gchar **argv = NULL;
 	GPtrArray *array = NULL;
@@ -698,7 +701,7 @@ gcm_calibrate_argyll_display_draw_and_measure (GcmCalibrateArgyll *calibrate_arg
 	}
 
 	/* get l-cd or c-rt */
-	type = gcm_calibrate_argyll_get_display_type (calibrate_argyll);
+	kind = gcm_calibrate_argyll_get_display_kind (calibrate_argyll);
 
 	/* TRANSLATORS: title, drawing means painting to the screen */
 	title = _("Drawing the patches");
@@ -714,9 +717,12 @@ gcm_calibrate_argyll_display_draw_and_measure (GcmCalibrateArgyll *calibrate_arg
 	array = g_ptr_array_new_with_free_func (g_free);
 
 	/* setup the command */
-	g_ptr_array_add (array, g_strdup ("-v9"));
+#ifdef FIXED_ARGYLL
+	g_ptr_array_add (array, g_strdup (command));
+#endif
+	g_ptr_array_add (array, g_strdup ("-v"));
 	g_ptr_array_add (array, g_strdup_printf ("-d%i", priv->display));
-	g_ptr_array_add (array, g_strdup_printf ("-y%c", type));
+	g_ptr_array_add (array, g_strdup_printf ("-y%c", kind));
 	g_ptr_array_add (array, g_strdup ("-k"));
 	g_ptr_array_add (array, g_strdup_printf ("%s.cal", basename));
 	g_ptr_array_add (array, g_strdup (basename));
@@ -827,7 +833,10 @@ gcm_calibrate_argyll_display_generate_profile (GcmCalibrateArgyll *calibrate_arg
 	array = g_ptr_array_new_with_free_func (g_free);
 
 	/* setup the command */
-	g_ptr_array_add (array, g_strdup ("-v9"));
+#ifdef FIXED_ARGYLL
+	g_ptr_array_add (array, g_strdup (command));
+#endif
+	g_ptr_array_add (array, g_strdup ("-v"));
 	g_ptr_array_add (array, g_strdup_printf ("-A%s", manufacturer));
 	g_ptr_array_add (array, g_strdup_printf ("-M%s", model));
 	g_ptr_array_add (array, g_strdup_printf ("-D%s", description_new));
@@ -1051,7 +1060,10 @@ gcm_calibrate_argyll_device_measure (GcmCalibrateArgyll *calibrate_argyll, GErro
 	filename = g_strdup_printf ("%s.tif", basename);
 
 	/* setup the command */
-	g_ptr_array_add (array, g_strdup ("-v9"));
+#ifdef FIXED_ARGYLL
+	g_ptr_array_add (array, g_strdup (command));
+#endif
+	g_ptr_array_add (array, g_strdup ("-v"));
 	g_ptr_array_add (array, g_strdup ("-p"));
 	g_ptr_array_add (array, g_strdup ("-a"));
 	g_ptr_array_add (array, g_strdup (filename));
@@ -1167,7 +1179,10 @@ gcm_calibrate_argyll_device_generate_profile (GcmCalibrateArgyll *calibrate_argy
 	array = g_ptr_array_new_with_free_func (g_free);
 
 	/* setup the command */
-	g_ptr_array_add (array, g_strdup ("-v9"));
+#ifdef FIXED_ARGYLL
+	g_ptr_array_add (array, g_strdup (command));
+#endif
+	g_ptr_array_add (array, g_strdup ("-v"));
 	g_ptr_array_add (array, g_strdup_printf ("-A%s", manufacturer));
 	g_ptr_array_add (array, g_strdup_printf ("-M%s", model));
 	g_ptr_array_add (array, g_strdup_printf ("-D%s", description_tmp));
@@ -1388,6 +1403,123 @@ out:
 	return ret;
 }
 
+/**
+ * gcm_calibrate_argyll_spotread_read_chart:
+ **/
+static gboolean
+gcm_calibrate_argyll_spotread_read_chart (GcmCalibrateArgyll *calibrate_argyll, GError **error)
+{
+	gboolean ret = TRUE;
+	GcmCalibrateArgyllPrivate *priv = calibrate_argyll->priv;
+	gchar *command = NULL;
+	gchar **argv = NULL;
+	GPtrArray *array = NULL;
+	gchar *working_path = NULL;
+	const gchar *title;
+	const gchar *message;
+
+	/* get shared data */
+	g_object_get (calibrate_argyll,
+		      "working-path", &working_path,
+		      NULL);
+
+	/* get correct name of the command */
+	command = gcm_calibrate_argyll_get_tool_filename ("spotread", error);
+	if (command == NULL) {
+		ret = FALSE;
+		goto out;
+	}
+
+	/* TRANSLATORS: title, setting up the photospectromiter */
+	title = _("Setting up device");
+	/* TRANSLATORS: dialog message */
+	message = _("Setting up the device to read a spot color…");
+
+	/* push new messages into the UI */
+	gcm_calibrate_dialog_show (priv->calibrate_dialog, GCM_CALIBRATE_DIALOG_TAB_GENERIC, title, message);
+	gcm_calibrate_dialog_set_show_button_ok (priv->calibrate_dialog, FALSE);
+	gcm_calibrate_dialog_set_show_expander (priv->calibrate_dialog, TRUE);
+
+	/* reset flag so we exit after the single spotread */
+	priv->done_spot_read = FALSE;
+
+	/* argument array */
+	array = g_ptr_array_new_with_free_func (g_free);
+
+	/* setup the command */
+#ifdef FIXED_ARGYLL
+	g_ptr_array_add (array, g_strdup (command));
+#endif
+	g_ptr_array_add (array, g_strdup ("-v"));
+	if (priv->done_calibrate)
+		g_ptr_array_add (array, g_strdup ("-N"));
+	argv = gcm_utils_ptr_array_to_strv (array);
+	gcm_calibrate_argyll_debug_argv (command, argv);
+
+	/* start up the command */
+	priv->state = GCM_CALIBRATE_ARGYLL_STATE_RUNNING;
+	vte_terminal_reset (VTE_TERMINAL(priv->terminal), TRUE, FALSE);
+	priv->child_pid = vte_terminal_fork_command (VTE_TERMINAL(priv->terminal), command, argv, NULL, working_path, FALSE, FALSE, FALSE);
+
+	/* wait until finished */
+	g_main_loop_run (priv->loop);
+
+	/* get result */
+	if (priv->response == GTK_RESPONSE_CANCEL) {
+		g_set_error_literal (error,
+				     GCM_CALIBRATE_ERROR,
+				     GCM_CALIBRATE_ERROR_USER_ABORT,
+				     "calibration was cancelled");
+		ret = FALSE;
+		goto out;
+	}
+	if (priv->response == GTK_RESPONSE_REJECT) {
+		gchar *vte_text;
+		vte_text = vte_terminal_get_text (VTE_TERMINAL(priv->terminal), NULL, NULL, NULL);
+		g_set_error (error,
+			     GCM_CALIBRATE_ERROR,
+			     GCM_CALIBRATE_ERROR_INTERNAL,
+			     "command failed to run successfully: %s", vte_text);
+		g_free (vte_text);
+		ret = FALSE;
+		goto out;
+	}
+out:
+	if (array != NULL)
+		g_ptr_array_unref (array);
+	g_free (working_path);
+	g_free (command);
+	g_strfreev (argv);
+	return ret;
+}
+
+/**
+ * gcm_calibrate_argyll_spotread:
+ **/
+static gboolean
+gcm_calibrate_argyll_spotread (GcmCalibrate *calibrate, GtkWindow *window, GError **error)
+{
+	GcmCalibrateArgyll *calibrate_argyll = GCM_CALIBRATE_ARGYLL(calibrate);
+	GcmCalibrateArgyllPrivate *priv = calibrate_argyll->priv;
+	gboolean ret;
+
+	/* set modal windows up correctly */
+	gcm_calibrate_dialog_set_move_window (priv->calibrate_dialog, FALSE);
+	gcm_calibrate_dialog_set_window (priv->calibrate_dialog, window);
+
+	/* step 3 */
+	ret = gcm_calibrate_argyll_spotread_read_chart (calibrate_argyll, error);
+	if (!ret)
+		goto out;
+
+	/* step 5 */
+	ret = gcm_calibrate_argyll_remove_temp_files (calibrate_argyll, error);
+	if (!ret)
+		goto out;
+out:
+	return ret;
+}
+
 /**
  * gcm_calibrate_argyll_get_colorimeter_target:
  **/
@@ -1463,6 +1595,9 @@ gcm_calibrate_argyll_display_generate_targets (GcmCalibrateArgyll *calibrate_arg
 	array = g_ptr_array_new_with_free_func (g_free);
 
 	/* setup the command */
+#ifdef FIXED_ARGYLL
+	g_ptr_array_add (array, g_strdup (command));
+#endif
 	g_ptr_array_add (array, g_strdup ("-v"));
 
 	/* target instrument */
@@ -1622,7 +1757,7 @@ gcm_calibrate_argyll_set_device_from_ti2 (GcmCalibrate *calibrate, const gchar *
 		device_ptr = device + 7;
 
 	/* set for calibration */
-	egg_debug ("setting device to '%s'", device_ptr);
+	egg_debug ("setting instrument to '%s'", device_ptr);
 	g_object_set (calibrate,
 		      "device", device_ptr,
 		      NULL);
@@ -1656,6 +1791,69 @@ out:
 	return paper_size;
 }
 
+/**
+ * gcm_calibrate_argyll_printer_convert_jpeg:
+ **/
+static gboolean
+gcm_calibrate_argyll_printer_convert_jpeg (GcmCalibrateArgyll *calibrate_argyll, GError **error)
+{
+	GDir *dir;
+	const gchar *filename;
+	gchar *filename_tiff;
+	gchar *filename_jpg;
+	guint len;
+	gboolean ret = TRUE;
+	gchar *working_path = NULL;
+	GdkPixbuf *pixbuf;
+
+	/* need to ask if we are printing now, or using old data */
+	g_object_get (calibrate_argyll,
+		      "working-path", &working_path,
+		      NULL);
+
+	dir = g_dir_open (working_path, 0, error);
+	if (dir == NULL) {
+		ret = FALSE;
+		goto out;
+	}
+
+	filename = g_dir_read_name (dir);
+	while (filename != NULL) {
+		if (g_str_has_suffix (filename, ".tif")) {
+
+			/* get both files */
+			filename_tiff = g_build_filename (working_path, filename, NULL);
+			filename_jpg = g_strdup (filename_tiff);
+
+			/* replace ending */
+			len = strlen (filename_jpg);
+			g_strlcpy (filename_jpg+len-4, ".jpg", 5);
+
+			/* convert from tiff to jpg */
+			egg_debug ("convert %s to %s", filename_tiff, filename_jpg);
+			pixbuf = gdk_pixbuf_new_from_file (filename_tiff, error);
+			if (pixbuf == NULL) {
+				ret = FALSE;
+				goto out;
+			}
+
+			/* try to save new file */
+			ret = gdk_pixbuf_save (pixbuf, filename_jpg, "jpeg", error, "quality", "100", NULL);
+			if (!ret)
+				goto out;
+			g_object_unref (pixbuf);
+			g_free (filename_tiff);
+			g_free (filename_jpg);
+		}
+		filename = g_dir_read_name (dir);
+	}
+out:
+	if (dir != NULL)
+		g_dir_close (dir);
+	g_free (working_path);
+	return ret;
+}
+
 /**
  * gcm_calibrate_argyll_printer:
  **/
@@ -1716,6 +1914,11 @@ gcm_calibrate_argyll_printer (GcmCalibrate *calibrate, GtkWindow *window, GError
 		if (!ret)
 			goto out;
 
+		/* convert to jpegs */
+		ret = gcm_calibrate_argyll_printer_convert_jpeg (GCM_CALIBRATE_ARGYLL (calibrate), error);
+		if (!ret)
+			goto out;
+
 		cmdline = g_strdup_printf ("nautilus \"%s\"", working_path);
 		egg_debug ("we need to open the directory we're using: %s", cmdline);
 		ret = g_spawn_command_line_async (cmdline, error);
@@ -1728,7 +1931,7 @@ gcm_calibrate_argyll_printer (GcmCalibrate *calibrate, GtkWindow *window, GError
 		title = _("Wait for the ink to dry");
 
 		/* TRANSLATORS: dialog message */
-		message = _("Please wait a few minutes for the ink to dry. Profiling damp ink will produce a poor profile and may damage your calibration device.");
+		message = _("Please wait a few minutes for the ink to dry. Profiling damp ink will produce a poor profile and may damage your color measuring instrument.");
 
 		/* push new messages into the UI */
 		gcm_calibrate_dialog_show (priv->calibrate_dialog, GCM_CALIBRATE_DIALOG_TAB_GENERIC, title, message);
@@ -1794,6 +1997,139 @@ out:
 	return ret;
 }
 
+/**
+ * gcm_calibrate_argyll_pixbuf_remove_alpha:
+ **/
+static GdkPixbuf *
+gcm_calibrate_argyll_pixbuf_remove_alpha (const GdkPixbuf *pixbuf)
+{
+	GdkPixbuf *new_pixbuf = NULL;
+	gint x, y;
+	guchar *src, *dest;
+
+	/* already no alpha */
+	if (!gdk_pixbuf_get_has_alpha (pixbuf)) {
+		new_pixbuf = gdk_pixbuf_copy (pixbuf);
+		goto out;
+	}
+
+	/* create new image, and copy RGB */
+	new_pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8,
+				     gdk_pixbuf_get_width (pixbuf),
+				     gdk_pixbuf_get_height (pixbuf));
+	for (y = 0; y < gdk_pixbuf_get_height (pixbuf); y++) {
+		src = gdk_pixbuf_get_pixels (pixbuf) + y * gdk_pixbuf_get_rowstride (pixbuf);
+		dest = gdk_pixbuf_get_pixels (new_pixbuf) + y * gdk_pixbuf_get_rowstride (new_pixbuf);
+
+		/* copy RGB from RGBA */
+		for (x = 0; x < gdk_pixbuf_get_width (pixbuf); x++) {
+			dest[0] = src[0];
+			dest[1] = src[1];
+			dest[2] = src[2];
+			src += 4;
+			dest += 3;
+		}
+	}
+out:
+	return new_pixbuf;
+}
+
+/**
+ * gcm_calibrate_argyll_check_and_remove_alpha:
+ **/
+static gboolean
+gcm_calibrate_argyll_check_and_remove_alpha (GcmCalibrateArgyll *calibrate_argyll, GError **error)
+{
+	gboolean ret = TRUE;
+	GdkPixbuf *pixbuf = NULL;
+	GdkPixbuf *pixbuf_new = NULL;
+	gchar *reference_image = NULL;
+	gchar *basename = NULL;
+	gchar *working_path = NULL;
+	gchar *filename = NULL;
+	const gchar *title;
+	GString *string = NULL;
+	GtkResponseType response;
+	GcmCalibrateArgyllPrivate *priv = calibrate_argyll->priv;
+
+	/* get shared data */
+	g_object_get (calibrate_argyll,
+		      "basename", &basename,
+		      "working-path", &working_path,
+		      NULL);
+
+	/* get copied filename */
+	filename = g_strdup_printf ("%s.tif", basename);
+	reference_image = g_build_filename (working_path, filename, NULL);
+
+	/* check to see if the file has any alpha channel */
+	pixbuf = gdk_pixbuf_new_from_file (reference_image, error);
+	if (pixbuf == NULL) {
+		ret = FALSE;
+		goto out;
+	}
+
+	/* plain RGB */
+	if (!gdk_pixbuf_get_has_alpha (pixbuf))
+		goto out;
+
+	/* TRANSLATORS: the supplied image contains an alpha channel which we have to strip out */
+	title = _("Image is not suitable without conversion");
+
+	/* TRANSLATORS: dialog message */
+	string = g_string_new (_("The supplied image contains an alpha channel which the profiling tools do not understand."));
+	g_string_append (string, "\n\n");
+
+	/* TRANSLATORS: dialog message */
+	g_string_append (string, _("It is normally safe to convert the image, although you should ensure that the generated profile is valid."));
+
+	/* push new messages into the UI */
+	gcm_calibrate_dialog_show (priv->calibrate_dialog, GCM_CALIBRATE_DIALOG_TAB_GENERIC, title, string->str);
+	gcm_calibrate_dialog_set_show_button_ok (priv->calibrate_dialog, TRUE);
+	gcm_calibrate_dialog_set_show_expander (priv->calibrate_dialog, FALSE);
+	/* TRANSLATORS: button text to convert the RGBA image into RGB */
+	gcm_calibrate_dialog_set_button_ok_id (priv->calibrate_dialog, _("Convert"));
+	response = gcm_calibrate_dialog_run (priv->calibrate_dialog);
+	if (response != GTK_RESPONSE_OK) {
+		gcm_calibrate_dialog_hide (priv->calibrate_dialog);
+		g_set_error_literal (error,
+				     GCM_CALIBRATE_ERROR,
+				     GCM_CALIBRATE_ERROR_USER_ABORT,
+				     "user did not convert RGBA into RGB");
+		ret = FALSE;
+		goto out;
+	}
+
+	/* remove the alpha channel */
+	pixbuf_new = gcm_calibrate_argyll_pixbuf_remove_alpha (pixbuf);
+	if (pixbuf_new == NULL) {
+		g_set_error_literal (error,
+				     GCM_CALIBRATE_ERROR,
+				     GCM_CALIBRATE_ERROR_INTERNAL,
+				     "failed to remove alpha channel");
+		ret = FALSE;
+		goto out;
+	}
+
+	/* save */
+	ret = gdk_pixbuf_save (pixbuf_new, reference_image, "tiff", error, NULL);
+	if (!ret)
+		goto out;
+
+out:
+	g_free (working_path);
+	g_free (filename);
+	g_free (basename);
+	g_free (reference_image);
+	if (string != NULL)
+		g_string_free (string, TRUE);
+	if (pixbuf != NULL)
+		g_object_unref (pixbuf);
+	if (pixbuf_new != NULL)
+		g_object_unref (pixbuf_new);
+	return ret;
+}
+
 /**
  * gcm_calibrate_argyll_device:
  **/
@@ -1810,11 +2146,11 @@ gcm_calibrate_argyll_device (GcmCalibrate *calibrate, GtkWindow *window, GError
 	gcm_calibrate_dialog_set_move_window (priv->calibrate_dialog, FALSE);
 	gcm_calibrate_dialog_set_window (priv->calibrate_dialog, window);
 
-	/* TRANSLATORS: title, hardware refers to a calibration device */
-	title = _("Set up device");
+	/* TRANSLATORS: title, instrument refers to a calibration device */
+	title = _("Set up instrument");
 
 	/* TRANSLATORS: dialog message */
-	message = _("Setting up device for use…");
+	message = _("Setting up the instrument for use…");
 
 	/* push new messages into the UI */
 	gcm_calibrate_dialog_show (priv->calibrate_dialog, GCM_CALIBRATE_DIALOG_TAB_GENERIC, title, message);
@@ -1826,6 +2162,11 @@ gcm_calibrate_argyll_device (GcmCalibrate *calibrate, GtkWindow *window, GError
 	if (!ret)
 		goto out;
 
+	/* step 1.5 */
+	ret = gcm_calibrate_argyll_check_and_remove_alpha (calibrate_argyll, error);
+	if (!ret)
+		goto out;
+
 	/* step 2 */
 	ret = gcm_calibrate_argyll_device_measure (calibrate_argyll, error);
 	if (!ret)
@@ -1901,8 +2242,8 @@ gcm_calibrate_argyll_interaction_attach (GcmCalibrateArgyll *calibrate_argyll)
 		goto out;
 	}
 
-	/* TRANSLATORS: title, device is a hardware color calibration sensor */
-	title = _("Please attach device");
+	/* TRANSLATORS: title, instrument is a hardware color calibration sensor */
+	title = _("Please attach instrument");
 
 	/* get the image, if we have one */
 	filename = gcm_calibrate_argyll_get_colorimeter_image_attach (calibrate_argyll);
@@ -1910,10 +2251,10 @@ gcm_calibrate_argyll_interaction_attach (GcmCalibrateArgyll *calibrate_argyll)
 	/* different messages with or without image */
 	if (filename != NULL) {
 		/* TRANSLATORS: dialog message, ask user to attach device, and there's an example image */
-		message = _("Please attach the hardware device to the center of the screen on the gray square like the image below.");
+		message = _("Please attach the measuring instrument to the center of the screen on the gray square like the image below.");
 	} else {
 		/* TRANSLATORS: dialog message, ask user to attach device */
-		message = _("Please attach the hardware device to the center of the screen on the gray square.");
+		message = _("Please attach the measuring instrument to the center of the screen on the gray square.");
 	}
 
 	/* block for a response */
@@ -1956,8 +2297,8 @@ gcm_calibrate_argyll_interaction_calibrate (GcmCalibrateArgyll *calibrate_argyll
 	const gchar *filename;
 	GcmCalibrateArgyllPrivate *priv = calibrate_argyll->priv;
 
-	/* TRANSLATORS: title, device is a hardware color calibration sensor */
-	title = _("Please configure device");
+	/* TRANSLATORS: title, instrument is a hardware color calibration sensor */
+	title = _("Please configure instrument");
 
 	/* block for a response */
 	egg_debug ("blocking waiting for user input: %s", title);
@@ -1967,10 +2308,10 @@ gcm_calibrate_argyll_interaction_calibrate (GcmCalibrateArgyll *calibrate_argyll
 
 	if (filename != NULL) {
 		/* TRANSLATORS: this is when the user has to change a setting on the sensor, and we're showing a picture */
-		message = _("Please set the device to calibration mode like the image below.");
+		message = _("Please set the measuring instrument to calibration mode like the image below.");
 	} else {
 		/* TRANSLATORS: this is when the user has to change a setting on the sensor */
-		message = _("Please set the device to calibration mode.");
+		message = _("Please set the measuring instrument to calibration mode.");
 	}
 
 	/* push new messages into the UI */
@@ -1992,6 +2333,9 @@ gcm_calibrate_argyll_interaction_calibrate (GcmCalibrateArgyll *calibrate_argyll
 	/* assume it's no longer on the window */
 	priv->already_on_window = FALSE;
 
+	/* assume it was done correctly */
+	priv->done_calibrate = TRUE;
+
 	/* set state */
 	priv->argyllcms_ok = " ";
 	priv->state = GCM_CALIBRATE_ARGYLL_STATE_WAITING_FOR_STDIN;
@@ -2008,8 +2352,8 @@ gcm_calibrate_argyll_interaction_surface (GcmCalibrateArgyll *calibrate_argyll)
 	const gchar *filename;
 	GcmCalibrateArgyllPrivate *priv = calibrate_argyll->priv;
 
-	/* TRANSLATORS: title, device is a hardware color calibration sensor */
-	title = _("Please configure device");
+	/* TRANSLATORS: title, instrument is a hardware color calibration sensor */
+	title = _("Please configure instrument");
 
 	/* block for a response */
 	egg_debug ("blocking waiting for user input: %s", title);
@@ -2019,10 +2363,10 @@ gcm_calibrate_argyll_interaction_surface (GcmCalibrateArgyll *calibrate_argyll)
 
 	if (filename != NULL) {
 		/* TRANSLATORS: this is when the user has to change a setting on the sensor, and we're showing a picture */
-		message = _("Please set the device to screen mode like the image below, and ensure it is attached to the screen.");
+		message = _("Please set the measuring instrument to screen mode like the image below.");
 	} else {
 		/* TRANSLATORS: this is when the user has to change a setting on the sensor */
-		message = _("Please set the device to screen mode, and ensure it is attached to the screen.");
+		message = _("Please set the measuring instrument to screen mode.");
 	}
 
 	/* push new messages into the UI */
@@ -2062,6 +2406,7 @@ gcm_calibrate_argyll_process_output_cmd (GcmCalibrateArgyll *calibrate_argyll, c
 	const gchar *message;
 	GString *string = NULL;
 	gchar *found;
+	gchar **split = NULL;
 	gboolean ret = TRUE;
 	GcmCalibrateArgyllPrivate *priv = calibrate_argyll->priv;
 
@@ -2135,6 +2480,27 @@ gcm_calibrate_argyll_process_output_cmd (GcmCalibrateArgyll *calibrate_argyll, c
 		goto out;
 	}
 
+	/* spot read result */
+	found = g_strstr_len (line, -1, "Result is XYZ");
+	if (found != NULL) {
+		GcmXyz *xyz;
+		egg_warning ("line=%s", line);
+		split = g_strsplit (line, " ", -1);
+		xyz = gcm_xyz_new ();
+		g_object_set (xyz,
+			      "cie-x", g_ascii_strtod (split[4], NULL),
+			      "cie-y", g_ascii_strtod (split[5], NULL),
+			      "cie-z", g_ascii_strtod (split[6], NULL),
+			      NULL);
+		g_object_set (calibrate_argyll,
+			      "xyz", xyz,
+			      NULL);
+		priv->done_spot_read = TRUE;
+		gcm_calibrate_dialog_response (priv->calibrate_dialog, GTK_RESPONSE_CANCEL);
+		g_object_unref (xyz);
+		goto out;
+	}
+
 	/* error */
 	found = g_strstr_len (line, -1, "Error - ");
 	if (found != NULL) {
@@ -2144,17 +2510,17 @@ gcm_calibrate_argyll_process_output_cmd (GcmCalibrateArgyll *calibrate_argyll, c
 
 		if (g_strstr_len (line, -1, "No PLD firmware pattern is available") != NULL) {
 			/* TRANSLATORS: message, no firmware is available */
-			message = _("No firmware is installed for this device.");
+			message = _("No firmware is installed for this instrument.");
 		} else if (g_strstr_len (line, -1, "Pattern match wasn't good enough") != NULL) {
 			/* TRANSLATORS: message, the image wasn't good enough */
 			message = _("The pattern match wasn't good enough. Ensure you have the correct type of target selected.");
 		} else if (g_strstr_len (line, -1, "Aprox. fwd matrix unexpectedly singular") != NULL ||
 			   g_strstr_len (line, -1, "Inverting aprox. fwd matrix failed") != NULL) {
 			/* TRANSLATORS: message, the sensor got no readings */
-			message = _("The colorimeter got no valid readings. Please ensure the aperture is fully open.");
+			message = _("The measuring instrument got no valid readings. Please ensure the aperture is fully open.");
 		} else if (g_strstr_len (line, -1, "Device or resource busy") != NULL) {
 			/* TRANSLATORS: message, the colorimeter has got confused */
-			message = _("The colorimeter device is busy and is not starting up. Please remove the USB plug and re-insert before trying to use this device.");
+			message = _("The measuring instrument is busy and is not starting up. Please remove the USB plug and re-insert before trying to use this device.");
 		} else {
 			message = found + 8;
 		}
@@ -2222,8 +2588,12 @@ gcm_calibrate_argyll_process_output_cmd (GcmCalibrateArgyll *calibrate_argyll, c
 	/* reading strip */
 	if (g_str_has_prefix (line, "(Warning) Seem to have read strip pass ")) {
 
+		/* find the strip we read, and the one we wanted */
+		split = g_strsplit (line, " ", -1);
+		g_strdelimit (split[10], "!", '\0');
+
 		/* TRANSLATORS: dialog title, where %s is a letter like 'A' */
-		title_str = g_strdup_printf (_("Read strip %c rather than %c!"), line[39], line[39+14]);
+		title_str = g_strdup_printf (_("Read strip %s rather than %s!"), split[7], split[10]);
 
 		string = g_string_new ("");
 
@@ -2250,6 +2620,38 @@ gcm_calibrate_argyll_process_output_cmd (GcmCalibrateArgyll *calibrate_argyll, c
 		goto out;
 	}
 
+	/* reading spot */
+	if (g_str_has_prefix (line, "Place instrument on spot to be measured")) {
+		if (!priv->done_spot_read)
+			vte_terminal_feed_child (VTE_TERMINAL(priv->terminal), " ", 1);
+		gcm_calibrate_dialog_hide (priv->calibrate_dialog);
+		goto out;
+	}
+
+
+	/* reading strip */
+	if (g_str_has_prefix (line, "Spot read failed due to misread")) {
+
+		/* TRANSLATORS: title, the calibration failed */
+		title = _("Device Error");
+
+		/* TRANSLATORS: message, the sample was not read correctly */
+		message = _("The device could not measure the color spot correctly.");
+
+		/* push new messages into the UI */
+		gcm_calibrate_dialog_show (priv->calibrate_dialog, GCM_CALIBRATE_DIALOG_TAB_GENERIC, title, message);
+		gcm_calibrate_dialog_set_show_button_ok (priv->calibrate_dialog, TRUE);
+		gcm_calibrate_dialog_set_show_expander (priv->calibrate_dialog, TRUE);
+
+		/* TRANSLATORS: button */
+		gcm_calibrate_dialog_set_button_ok_id (priv->calibrate_dialog, _("Retry"));
+
+		/* set state */
+		priv->argyllcms_ok = " ";
+		priv->state = GCM_CALIBRATE_ARGYLL_STATE_WAITING_FOR_STDIN;
+		goto out;
+	}
+
 	/* reading strip */
 	if (g_str_has_prefix (line, "Ready to read strip pass ")) {
 
@@ -2285,6 +2687,7 @@ gcm_calibrate_argyll_process_output_cmd (GcmCalibrateArgyll *calibrate_argyll, c
 	egg_warning ("VTE: could not screenscrape: %s", line);
 out:
 	g_free (title_str);
+	g_strfreev (split);
 	if (string != NULL)
 		g_string_free (string, TRUE);
 	return ret;
@@ -2388,6 +2791,9 @@ gcm_calibrate_argyll_response_cb (GtkWidget *widget, GtkResponseType response, G
 			priv->state = GCM_CALIBRATE_ARGYLL_STATE_RUNNING;
 		}
 
+		/* hide the window */
+		gcm_calibrate_dialog_hide (priv->calibrate_dialog);
+
 		/* stop loop */
 		if (g_main_loop_is_running (priv->loop))
 			g_main_loop_quit (priv->loop);
@@ -2491,6 +2897,7 @@ gcm_calibrate_argyll_class_init (GcmCalibrateArgyllClass *klass)
 	parent_class->calibrate_display = gcm_calibrate_argyll_display;
 	parent_class->calibrate_device = gcm_calibrate_argyll_device;
 	parent_class->calibrate_printer = gcm_calibrate_argyll_printer;
+	parent_class->calibrate_spotread = gcm_calibrate_argyll_spotread;
 
 	g_type_class_add_private (klass, sizeof (GcmCalibrateArgyllPrivate));
 }
@@ -2501,14 +2908,13 @@ gcm_calibrate_argyll_class_init (GcmCalibrateArgyllClass *klass)
 static void
 gcm_calibrate_argyll_init (GcmCalibrateArgyll *calibrate_argyll)
 {
-	gchar *precision;
-
 	calibrate_argyll->priv = GCM_CALIBRATE_ARGYLL_GET_PRIVATE (calibrate_argyll);
 	calibrate_argyll->priv->child_pid = -1;
 	calibrate_argyll->priv->loop = g_main_loop_new (NULL, FALSE);
 	calibrate_argyll->priv->vte_previous_row = 0;
 	calibrate_argyll->priv->vte_previous_col = 0;
 	calibrate_argyll->priv->already_on_window = FALSE;
+	calibrate_argyll->priv->done_calibrate = FALSE;
 	calibrate_argyll->priv->state = GCM_CALIBRATE_ARGYLL_STATE_IDLE;
 	calibrate_argyll->priv->print = gcm_print_new ();
 	g_signal_connect (calibrate_argyll->priv->print, "status-changed",
@@ -2517,9 +2923,6 @@ gcm_calibrate_argyll_init (GcmCalibrateArgyll *calibrate_argyll)
 	g_signal_connect (calibrate_argyll->priv->calibrate_dialog, "response",
 			  G_CALLBACK (gcm_calibrate_argyll_response_cb), calibrate_argyll);
 
-	/* use GConf to get defaults */
-	calibrate_argyll->priv->gconf_client = gconf_client_get_default ();
-
 	/* get screen */
 	calibrate_argyll->priv->screen = gcm_screen_new ();
 
@@ -2532,11 +2935,6 @@ gcm_calibrate_argyll_init (GcmCalibrateArgyll *calibrate_argyll)
 			  G_CALLBACK (gcm_calibrate_argyll_cursor_moved_cb), calibrate_argyll);
 	gcm_calibrate_dialog_pack_details (calibrate_argyll->priv->calibrate_dialog,
 					   calibrate_argyll->priv->terminal);
-
-	/* get default precision */
-	precision = gconf_client_get_string (calibrate_argyll->priv->gconf_client, GCM_SETTINGS_CALIBRATION_LENGTH, NULL);
-	calibrate_argyll->priv->precision = gcm_calibrate_argyll_precision_from_string (precision);
-	g_free (precision);
 }
 
 /**
@@ -2562,7 +2960,6 @@ gcm_calibrate_argyll_finalize (GObject *object)
 
 	g_main_loop_unref (priv->loop);
 	g_object_unref (priv->screen);
-	g_object_unref (priv->gconf_client);
 	g_object_unref (priv->calibrate_dialog);
 	g_object_unref (priv->print);
 
