@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2009 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2009-2010 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -54,6 +54,7 @@ struct _GcmCalibrateDialogPrivate
 	GcmCalibrateDeviceKind		 device_kind;
 	GcmCalibratePrintKind		 print_kind;
 	GcmCalibrateReferenceKind	 reference_kind;
+	GcmCalibratePrecision		 precision;
 	GtkResponseType			 response;
 	GMainLoop			*loop;
 	gboolean			 move_window;
@@ -69,6 +70,7 @@ enum {
 	PROP_DEVICE_KIND,
 	PROP_PRINT_KIND,
 	PROP_REFERENCE_KIND,
+	PROP_PRECISION,
 	PROP_LAST
 };
 
@@ -86,10 +88,10 @@ static guint signals[SIGNAL_LAST] = { 0 };
 G_DEFINE_TYPE (GcmCalibrateDialog, gcm_calibrate_dialog, G_TYPE_OBJECT)
 
 /**
- * gcm_calibrate_dialog_emit_response:
+ * gcm_calibrate_dialog_response:
  **/
-static void
-gcm_calibrate_dialog_emit_response (GcmCalibrateDialog *calibrate_dialog, GtkResponseType response)
+void
+gcm_calibrate_dialog_response (GcmCalibrateDialog *calibrate_dialog, GtkResponseType response)
 {
 	calibrate_dialog->priv->response = response;
 	g_signal_emit (calibrate_dialog, signals[SIGNAL_RESPONSE], 0, response);
@@ -106,7 +108,7 @@ static void
 gcm_calibrate_dialog_button_clicked_lcd_cb (GtkWidget *widget, GcmCalibrateDialog *calibrate_dialog)
 {
 	calibrate_dialog->priv->device_kind = GCM_CALIBRATE_DEVICE_KIND_LCD;
-	gcm_calibrate_dialog_emit_response (calibrate_dialog, GTK_RESPONSE_OK);
+	gcm_calibrate_dialog_response (calibrate_dialog, GTK_RESPONSE_OK);
 }
 
 /**
@@ -116,7 +118,7 @@ static void
 gcm_calibrate_dialog_button_clicked_crt_cb (GtkWidget *widget, GcmCalibrateDialog *calibrate_dialog)
 {
 	calibrate_dialog->priv->device_kind = GCM_CALIBRATE_DEVICE_KIND_CRT;
-	gcm_calibrate_dialog_emit_response (calibrate_dialog, GTK_RESPONSE_OK);
+	gcm_calibrate_dialog_response (calibrate_dialog, GTK_RESPONSE_OK);
 }
 
 /**
@@ -126,7 +128,7 @@ static void
 gcm_calibrate_dialog_button_clicked_projector_cb (GtkWidget *widget, GcmCalibrateDialog *calibrate_dialog)
 {
 	calibrate_dialog->priv->device_kind = GCM_CALIBRATE_DEVICE_KIND_PROJECTOR;
-	gcm_calibrate_dialog_emit_response (calibrate_dialog, GTK_RESPONSE_OK);
+	gcm_calibrate_dialog_response (calibrate_dialog, GTK_RESPONSE_OK);
 }
 
 /**
@@ -136,7 +138,7 @@ static void
 gcm_calibrate_dialog_button_clicked_print_local_cb (GtkWidget *widget, GcmCalibrateDialog *calibrate_dialog)
 {
 	calibrate_dialog->priv->print_kind = GCM_CALIBRATE_PRINT_KIND_LOCAL;
-	gcm_calibrate_dialog_emit_response (calibrate_dialog, GTK_RESPONSE_OK);
+	gcm_calibrate_dialog_response (calibrate_dialog, GTK_RESPONSE_OK);
 }
 
 /**
@@ -146,7 +148,7 @@ static void
 gcm_calibrate_dialog_button_clicked_print_generate_cb (GtkWidget *widget, GcmCalibrateDialog *calibrate_dialog)
 {
 	calibrate_dialog->priv->print_kind = GCM_CALIBRATE_PRINT_KIND_GENERATE;
-	gcm_calibrate_dialog_emit_response (calibrate_dialog, GTK_RESPONSE_OK);
+	gcm_calibrate_dialog_response (calibrate_dialog, GTK_RESPONSE_OK);
 }
 
 /**
@@ -156,7 +158,37 @@ static void
 gcm_calibrate_dialog_button_clicked_print_analyze_cb (GtkWidget *widget, GcmCalibrateDialog *calibrate_dialog)
 {
 	calibrate_dialog->priv->print_kind = GCM_CALIBRATE_PRINT_KIND_ANALYZE;
-	gcm_calibrate_dialog_emit_response (calibrate_dialog, GTK_RESPONSE_OK);
+	gcm_calibrate_dialog_response (calibrate_dialog, GTK_RESPONSE_OK);
+}
+
+/**
+ * gcm_calibrate_dialog_button_clicked_precision_short_cb:
+ **/
+static void
+gcm_calibrate_dialog_button_clicked_precision_short_cb (GtkWidget *widget, GcmCalibrateDialog *calibrate_dialog)
+{
+	calibrate_dialog->priv->precision = GCM_CALIBRATE_PRECISION_SHORT;
+	gcm_calibrate_dialog_response (calibrate_dialog, GTK_RESPONSE_OK);
+}
+
+/**
+ * gcm_calibrate_dialog_button_clicked_precision_normal_cb:
+ **/
+static void
+gcm_calibrate_dialog_button_clicked_precision_normal_cb (GtkWidget *widget, GcmCalibrateDialog *calibrate_dialog)
+{
+	calibrate_dialog->priv->precision = GCM_CALIBRATE_PRECISION_NORMAL;
+	gcm_calibrate_dialog_response (calibrate_dialog, GTK_RESPONSE_OK);
+}
+
+/**
+ * gcm_calibrate_dialog_button_clicked_precision_long_cb:
+ **/
+static void
+gcm_calibrate_dialog_button_clicked_precision_long_cb (GtkWidget *widget, GcmCalibrateDialog *calibrate_dialog)
+{
+	calibrate_dialog->priv->precision = GCM_CALIBRATE_PRECISION_LONG;
+	gcm_calibrate_dialog_response (calibrate_dialog, GTK_RESPONSE_OK);
 }
 
 /**
@@ -165,7 +197,7 @@ gcm_calibrate_dialog_button_clicked_print_analyze_cb (GtkWidget *widget, GcmCali
 static void
 gcm_calibrate_dialog_button_clicked_ok_cb (GtkWidget *widget, GcmCalibrateDialog *calibrate_dialog)
 {
-	gcm_calibrate_dialog_emit_response (calibrate_dialog, GTK_RESPONSE_OK);
+	gcm_calibrate_dialog_response (calibrate_dialog, GTK_RESPONSE_OK);
 }
 
 /**
@@ -174,7 +206,7 @@ gcm_calibrate_dialog_button_clicked_ok_cb (GtkWidget *widget, GcmCalibrateDialog
 static void
 gcm_calibrate_dialog_button_clicked_cancel_cb (GtkWidget *widget, GcmCalibrateDialog *calibrate_dialog)
 {
-	gcm_calibrate_dialog_emit_response (calibrate_dialog, GTK_RESPONSE_CANCEL);
+	gcm_calibrate_dialog_response (calibrate_dialog, GTK_RESPONSE_CANCEL);
 }
 
 /**
@@ -246,11 +278,20 @@ gcm_calibrate_dialog_set_window	(GcmCalibrateDialog *calibrate_dialog, GtkWindow
 {
 	GtkWidget *widget;
 	GcmCalibrateDialogPrivate *priv = calibrate_dialog->priv;
-	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "dialog_calibrate"));
 
-	/* run dialog */
-	if (window != NULL)
-		gtk_window_set_transient_for (GTK_WINDOW (widget), window);
+	/* do nothing, we can't unparent */
+	if (window == NULL)
+		return;
+
+	/* ensure it's not the same thing */
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "dialog_calibrate"));
+	if (GTK_WINDOW (widget) == window) {
+		egg_warning ("trying to set parent to self!");
+		return;
+	}
+
+	/* make modal */
+	gtk_window_set_transient_for (GTK_WINDOW (widget), window);
 }
 
 /**
@@ -322,6 +363,8 @@ gcm_calibrate_dialog_show (GcmCalibrateDialog		*calibrate_dialog,
 	gtk_widget_set_visible (widget, (tab == GCM_CALIBRATE_DIALOG_TAB_GENERIC));
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "vbox_print_mode"));
 	gtk_widget_set_visible (widget, (tab == GCM_CALIBRATE_DIALOG_TAB_PRINT_MODE));
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "vbox_precision"));
+	gtk_widget_set_visible (widget, (tab == GCM_CALIBRATE_DIALOG_TAB_PRECISION));
 
 	/* reset */
 	gcm_calibrate_dialog_set_image_filename (calibrate_dialog, NULL);
@@ -469,7 +512,7 @@ gcm_calibrate_dialog_set_move_window (GcmCalibrateDialog *calibrate_dialog, gboo
 static gboolean
 gcm_calibrate_dialog_delete_event_cb (GtkWidget *widget, GdkEvent *event, GcmCalibrateDialog *calibrate_dialog)
 {
-	gcm_calibrate_dialog_emit_response (calibrate_dialog, GTK_RESPONSE_CANCEL);
+	gcm_calibrate_dialog_response (calibrate_dialog, GTK_RESPONSE_CANCEL);
 	return FALSE;
 }
 
@@ -482,7 +525,7 @@ gcm_calibrate_dialog_reference_kind_to_thumbnail_image_filename (GcmCalibrateRef
 	if (kind == GCM_CALIBRATE_REFERENCE_KIND_CMP_DIGITAL_TARGET_3)
 		return "CMP-DigitalTarget3.png";
 	if (kind == GCM_CALIBRATE_REFERENCE_KIND_CMP_DT_003)
-		return NULL;
+		return "CMP_DT_003.png";
 	if (kind == GCM_CALIBRATE_REFERENCE_KIND_COLOR_CHECKER)
 		return "ColorChecker24.png";
 	if (kind == GCM_CALIBRATE_REFERENCE_KIND_COLOR_CHECKER_DC)
@@ -615,6 +658,9 @@ gcm_calibrate_dialog_get_property (GObject *object, guint prop_id, GValue *value
 	case PROP_PRINT_KIND:
 		g_value_set_uint (value, priv->print_kind);
 		break;
+	case PROP_PRECISION:
+		g_value_set_uint (value, priv->precision);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -670,6 +716,14 @@ gcm_calibrate_dialog_class_init (GcmCalibrateDialogClass *klass)
 				   G_PARAM_READABLE);
 	g_object_class_install_property (object_class, PROP_PRINT_KIND, pspec);
 
+	/**
+	 * GcmCalibrateDialog:precision:
+	 */
+	pspec = g_param_spec_uint ("precision", NULL, NULL,
+				   0, G_MAXUINT, 0,
+				   G_PARAM_READABLE);
+	g_object_class_install_property (object_class, PROP_PRECISION, pspec);
+
 	/**
 	 * GcmCalibrateDialog::response:
 	 **/
@@ -699,6 +753,7 @@ gcm_calibrate_dialog_init (GcmCalibrateDialog *calibrate_dialog)
 	calibrate_dialog->priv->device_kind = GCM_CALIBRATE_DEVICE_KIND_UNKNOWN;
 	calibrate_dialog->priv->print_kind = GCM_CALIBRATE_PRINT_KIND_UNKNOWN;
 	calibrate_dialog->priv->reference_kind = GCM_CALIBRATE_REFERENCE_KIND_UNKNOWN;
+	calibrate_dialog->priv->precision = GCM_CALIBRATE_PRECISION_UNKNOWN;
 	calibrate_dialog->priv->move_window = FALSE;
 	calibrate_dialog->priv->loop = g_main_loop_new (NULL, FALSE);
 	calibrate_dialog->priv->cached_dialogs = g_ptr_array_new_with_free_func ((GDestroyNotify)gcm_calibrate_dialog_dialog_free);
@@ -738,6 +793,15 @@ gcm_calibrate_dialog_init (GcmCalibrateDialog *calibrate_dialog)
 	widget = GTK_WIDGET (gtk_builder_get_object (calibrate_dialog->priv->builder, "button_print_analyze"));
 	g_signal_connect (widget, "clicked",
 			  G_CALLBACK (gcm_calibrate_dialog_button_clicked_print_analyze_cb), calibrate_dialog);
+	widget = GTK_WIDGET (gtk_builder_get_object (calibrate_dialog->priv->builder, "button_precision_short"));
+	g_signal_connect (widget, "clicked",
+			  G_CALLBACK (gcm_calibrate_dialog_button_clicked_precision_short_cb), calibrate_dialog);
+	widget = GTK_WIDGET (gtk_builder_get_object (calibrate_dialog->priv->builder, "button_precision_normal"));
+	g_signal_connect (widget, "clicked",
+			  G_CALLBACK (gcm_calibrate_dialog_button_clicked_precision_normal_cb), calibrate_dialog);
+	widget = GTK_WIDGET (gtk_builder_get_object (calibrate_dialog->priv->builder, "button_precision_long"));
+	g_signal_connect (widget, "clicked",
+			  G_CALLBACK (gcm_calibrate_dialog_button_clicked_precision_long_cb), calibrate_dialog);
 
 	widget = GTK_WIDGET (gtk_builder_get_object (calibrate_dialog->priv->builder, "image_target"));
 	gtk_widget_set_size_request (widget, 200, 140);
