@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2009 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2009-2010 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -32,8 +32,12 @@
 #include <gtk/gtk.h>
 #include <tiff.h>
 #include <tiffio.h>
+#include <gconf/gconf-client.h>
 
 #include "gcm-calibrate.h"
+#include "gcm-xyz.h"
+#include "gcm-dmi.h"
+#include "gcm-device-xrandr.h"
 #include "gcm-utils.h"
 #include "gcm-brightness.h"
 #include "gcm-colorimeter.h"
@@ -52,13 +56,16 @@ static void     gcm_calibrate_finalize	(GObject     *object);
  **/
 struct _GcmCalibratePrivate
 {
+	GcmDmi				*dmi;
 	GcmColorimeter			*colorimeter;
 	GcmCalibrateReferenceKind	 reference_kind;
-	GcmCalibrateDeviceKind		 device_kind;
+	GcmCalibrateDeviceKind		 calibrate_device_kind;
 	GcmCalibratePrintKind		 print_kind;
+	GcmCalibratePrecision		 precision;
 	GcmColorimeterKind		 colorimeter_kind;
 	GcmCalibrateDialog		*calibrate_dialog;
-	GcmDeviceTypeEnum		 device_type;
+	GcmDeviceKind			 device_kind;
+	GcmXyz				*xyz;
 	gchar				*output_name;
 	gchar				*filename_source;
 	gchar				*filename_reference;
@@ -70,6 +77,7 @@ struct _GcmCalibratePrivate
 	gchar				*serial;
 	gchar				*device;
 	gchar				*working_path;
+	GConfClient			*gconf_client;
 };
 
 enum {
@@ -81,20 +89,40 @@ enum {
 	PROP_DEVICE,
 	PROP_MANUFACTURER,
 	PROP_REFERENCE_KIND,
-	PROP_DEVICE_KIND,
+	PROP_CALIBRATE_DEVICE_KIND,
 	PROP_PRINT_KIND,
-	PROP_DEVICE_TYPE,
+	PROP_DEVICE_KIND,
 	PROP_COLORIMETER_KIND,
 	PROP_OUTPUT_NAME,
 	PROP_FILENAME_SOURCE,
 	PROP_FILENAME_REFERENCE,
 	PROP_FILENAME_RESULT,
 	PROP_WORKING_PATH,
+	PROP_PRECISION,
+	PROP_XYZ,
 	PROP_LAST
 };
 
 G_DEFINE_TYPE (GcmCalibrate, gcm_calibrate, G_TYPE_OBJECT)
 
+/**
+ * gcm_calibrate_precision_from_string:
+ **/
+static GcmCalibratePrecision
+gcm_calibrate_precision_from_string (const gchar *string)
+{
+	if (g_strcmp0 (string, "short") == 0)
+		return GCM_CALIBRATE_PRECISION_SHORT;
+	if (g_strcmp0 (string, "normal") == 0)
+		return GCM_CALIBRATE_PRECISION_NORMAL;
+	if (g_strcmp0 (string, "long") == 0)
+		return GCM_CALIBRATE_PRECISION_LONG;
+	if (g_strcmp0 (string, "ask") == 0)
+		return GCM_CALIBRATE_PRECISION_UNKNOWN;
+	egg_warning ("failed to convert to precision: %s", string);
+	return GCM_CALIBRATE_PRECISION_UNKNOWN;
+}
+
 /**
  * gcm_calibrate_get_model_fallback:
  **/
@@ -169,6 +197,15 @@ gcm_calibrate_get_time (void)
 	return text;
 }
 
+/**
+ * gcm_calibrate_get_filename_result:
+ **/
+const gchar *
+gcm_calibrate_get_filename_result (GcmCalibrate *calibrate)
+{
+	return calibrate->priv->filename_result;
+}
+
 /**
  * gcm_calibrate_set_basename:
  **/
@@ -225,26 +262,36 @@ gcm_calibrate_set_basename (GcmCalibrate *calibrate)
 gboolean
 gcm_calibrate_set_from_device (GcmCalibrate *calibrate, GcmDevice *device, GError **error)
 {
+	gboolean lcd_internal;
 	gboolean ret = TRUE;
-	gchar *native_device = NULL;
-	gchar *manufacturer = NULL;
-	gchar *model = NULL;
-	gchar *description = NULL;
-	gchar *serial = NULL;
-	GcmDeviceTypeEnum type;
+	const gchar *native_device = NULL;
+	const gchar *manufacturer = NULL;
+	const gchar *model = NULL;
+	const gchar *description = NULL;
+	const gchar *serial = NULL;
+	GcmDeviceKind kind;
+	GcmCalibratePrivate *priv = calibrate->priv;
 
 	/* get the device */
-	g_object_get (device,
-		      "type", &type,
-		      "serial", &serial,
-		      "model", &model,
-		      "title", &description,
-		      "manufacturer", &manufacturer,
-		      NULL);
+	kind = gcm_device_get_kind (device);
+	serial = gcm_device_get_serial (device);
+	model = gcm_device_get_model (device);
+	description = gcm_device_get_title (device);
+	manufacturer = gcm_device_get_manufacturer (device);
+
+	/* if we're a laptop, maybe use the dmi data instead */
+	if (kind == GCM_DEVICE_KIND_DISPLAY) {
+		native_device = gcm_device_xrandr_get_native_device (GCM_DEVICE_XRANDR (device));
+		lcd_internal = gcm_utils_output_is_lcd_internal (native_device);
+		if (lcd_internal) {
+			model = gcm_dmi_get_name (priv->dmi);
+			manufacturer = gcm_dmi_get_vendor (priv->dmi);
+		}
+	}
 
 	/* set the proper values */
 	g_object_set (calibrate,
-		      "device-type", type,
+		      "device-kind", kind,
 		      "model", model,
 		      "description", description,
 		      "manufacturer", manufacturer,
@@ -255,10 +302,8 @@ gcm_calibrate_set_from_device (GcmCalibrate *calibrate, GcmDevice *device, GErro
 	gcm_calibrate_set_basename (calibrate);
 
 	/* display specific properties */
-	if (type == GCM_DEVICE_TYPE_ENUM_DISPLAY) {
-		g_object_get (device,
-			      "native-device", &native_device,
-			      NULL);
+	if (kind == GCM_DEVICE_KIND_DISPLAY) {
+		native_device = gcm_device_xrandr_get_native_device (GCM_DEVICE_XRANDR (device));
 		if (native_device == NULL) {
 			g_set_error (error,
 				     GCM_CALIBRATE_ERROR,
@@ -273,11 +318,6 @@ gcm_calibrate_set_from_device (GcmCalibrate *calibrate, GcmDevice *device, GErro
 	}
 
 out:
-	g_free (native_device);
-	g_free (manufacturer);
-	g_free (model);
-	g_free (description);
-	g_free (serial);
 	return ret;
 }
 
@@ -330,10 +370,10 @@ out:
 }
 
 /**
- * gcm_calibrate_get_display_type:
+ * gcm_calibrate_get_display_kind:
  **/
 static gboolean
-gcm_calibrate_get_display_type (GcmCalibrate *calibrate, GtkWindow *window, GError **error)
+gcm_calibrate_get_display_kind (GcmCalibrate *calibrate, GtkWindow *window, GError **error)
 {
 	gboolean ret = TRUE;
 	const gchar *title;
@@ -364,16 +404,16 @@ gcm_calibrate_get_display_type (GcmCalibrate *calibrate, GtkWindow *window, GErr
 	}
 
 	/* copy */
-	g_object_get (priv->calibrate_dialog, "device-kind", &priv->device_kind, NULL);
+	g_object_get (priv->calibrate_dialog, "device-kind", &priv->calibrate_device_kind, NULL);
 
 	/* can this device support projectors? */
-	if (priv->device_kind == GCM_CALIBRATE_DEVICE_KIND_PROJECTOR &&
+	if (priv->calibrate_device_kind == GCM_CALIBRATE_DEVICE_KIND_PROJECTOR &&
 	    !gcm_colorimeter_supports_projector (priv->colorimeter)) {
 		/* TRANSLATORS: title, the hardware calibration device does not support projectors */
-		title = _("Could not calibrate using this colorimeter device");
+		title = _("Could not calibrate and profile using this color measuring instrument");
 
 		/* TRANSLATORS: dialog message */
-		message = _("This colorimeter device is not designed to support profiling projectors.");
+		message = _("This color measuring instrument is not designed to support calibration and profiling projectors.");
 
 		/* ask the user again */
 		gcm_calibrate_dialog_show (priv->calibrate_dialog, GCM_CALIBRATE_DIALOG_TAB_GENERIC, title, message);
@@ -418,6 +458,88 @@ gcm_calibrate_set_working_path (GcmCalibrate *calibrate, GError **error)
 	return ret;
 }
 
+
+/**
+ * gcm_calibrate_get_precision:
+ **/
+static GcmCalibratePrecision
+gcm_calibrate_get_precision (GcmCalibrate *calibrate, GError **error)
+{
+	GcmCalibratePrecision precision = GCM_CALIBRATE_PRECISION_UNKNOWN;
+	const gchar *title;
+	GString *string;
+	GtkResponseType response;
+	GcmCalibratePrivate *priv = calibrate->priv;
+
+	string = g_string_new ("");
+
+	/* TRANSLATORS: dialog title */
+	title = _("Profile Precision");
+
+	/* TRANSLATORS: this is the message body for the chart selection */
+	g_string_append (string, _("A higher precision profile provides higher accuracy in color matching but requires more time for reading the color patches."));
+
+	/* TRANSLATORS: this is the message body for the chart selection */
+	g_string_append_printf (string, "\n%s", _("For a typical workflow, a normal precision profile is sufficient."));
+
+	/* printer specific options */
+	if (priv->device_kind == GCM_DEVICE_KIND_PRINTER) {
+		/* TRANSLATORS: dialog message, preface */
+		g_string_append_printf (string, "\n%s", _("The high precision profile also requires more paper and printer ink."));
+	}
+
+	/* push new messages into the UI */
+	gcm_calibrate_dialog_show (priv->calibrate_dialog, GCM_CALIBRATE_DIALOG_TAB_PRECISION, title, string->str);
+	gcm_calibrate_dialog_set_show_button_ok (priv->calibrate_dialog, FALSE);
+	gcm_calibrate_dialog_set_show_expander (priv->calibrate_dialog, FALSE);
+	response = gcm_calibrate_dialog_run (priv->calibrate_dialog);
+	if (response != GTK_RESPONSE_OK) {
+		gcm_calibrate_dialog_hide (priv->calibrate_dialog);
+		g_set_error_literal (error,
+				     GCM_CALIBRATE_ERROR,
+				     GCM_CALIBRATE_ERROR_USER_ABORT,
+				     "user did not choose precision type and ask is specified in GConf");
+		goto out;
+	}
+
+	/* copy */
+	g_object_get (priv->calibrate_dialog, "precision", &precision, NULL);
+out:
+	if (string != NULL)
+		g_string_free (string, TRUE);
+	return precision;
+}
+
+/**
+ * gcm_calibrate_spotread:
+ **/
+gboolean
+gcm_calibrate_spotread (GcmCalibrate *calibrate, GtkWindow *window, GError **error)
+{
+	GcmCalibrateClass *klass = GCM_CALIBRATE_GET_CLASS (calibrate);
+	gboolean ret;
+
+	/* set the per-profile filename */
+	ret = gcm_calibrate_set_working_path (calibrate, error);
+	if (!ret)
+		goto out;
+
+	/* coldplug source */
+	if (klass->calibrate_spotread == NULL) {
+		ret = FALSE;
+		g_set_error_literal (error,
+				     GCM_CALIBRATE_ERROR,
+				     GCM_CALIBRATE_ERROR_INTERNAL,
+				     "no klass support");
+		goto out;
+	}
+
+	/* proxy */
+	ret = klass->calibrate_spotread (calibrate, window, error);
+out:
+	return ret;
+}
+
 /**
  * gcm_calibrate_display:
  **/
@@ -433,6 +555,7 @@ gcm_calibrate_display (GcmCalibrate *calibrate, GtkWindow *window, GError **erro
 	guint percentage = G_MAXUINT;
 	GtkResponseType response;
 	GError *error_tmp = NULL;
+	gchar *precision = NULL;
 	GcmCalibratePrivate *priv = calibrate->priv;
 
 	/* coldplug source */
@@ -475,12 +598,23 @@ gcm_calibrate_display (GcmCalibrate *calibrate, GtkWindow *window, GError **erro
 		      NULL);
 
 	/* this wasn't previously set */
-	if (priv->device_kind == GCM_CALIBRATE_DEVICE_KIND_UNKNOWN) {
-		ret = gcm_calibrate_get_display_type (calibrate, window, error);
+	if (priv->calibrate_device_kind == GCM_CALIBRATE_DEVICE_KIND_UNKNOWN) {
+		ret = gcm_calibrate_get_display_kind (calibrate, window, error);
 		if (!ret)
 			goto out;
 	}
 
+	/* get default precision */
+	precision = gconf_client_get_string (priv->gconf_client, GCM_SETTINGS_CALIBRATION_LENGTH, NULL);
+	priv->precision = gcm_calibrate_precision_from_string (precision);
+	if (priv->precision == GCM_CALIBRATE_PRECISION_UNKNOWN) {
+		priv->precision = gcm_calibrate_get_precision (calibrate, error);
+		if (priv->precision == GCM_CALIBRATE_PRECISION_UNKNOWN) {
+			ret = FALSE;
+			goto out;
+		}
+	}
+
 	/* show a warning for external monitors */
 	ret = gcm_utils_output_is_lcd_internal (priv->output_name);
 	if (!ret) {
@@ -571,6 +705,7 @@ out:
 		g_object_unref (brightness);
 	if (string != NULL)
 		g_string_free (string, TRUE);
+	g_free (precision);
 	return ret;
 }
 
@@ -758,6 +893,7 @@ gcm_calibrate_printer (GcmCalibrate *calibrate, GtkWindow *window, GError **erro
 	gchar *ptr;
 	GtkWindow *window_tmp;
 	GtkResponseType response;
+	gchar *precision = NULL;
 	GcmCalibrateClass *klass = GCM_CALIBRATE_GET_CLASS (calibrate);
 	GcmCalibratePrivate *priv = calibrate->priv;
 
@@ -782,6 +918,17 @@ gcm_calibrate_printer (GcmCalibrate *calibrate, GtkWindow *window, GError **erro
 		goto out;
 	}
 
+	/* get default precision */
+	precision = gconf_client_get_string (priv->gconf_client, GCM_SETTINGS_CALIBRATION_LENGTH, NULL);
+	priv->precision = gcm_calibrate_precision_from_string (precision);
+	if (priv->precision == GCM_CALIBRATE_PRECISION_UNKNOWN) {
+		priv->precision = gcm_calibrate_get_precision (calibrate, error);
+		if (priv->precision == GCM_CALIBRATE_PRECISION_UNKNOWN) {
+			ret = FALSE;
+			goto out;
+		}
+	}
+
 	/* copy */
 	g_object_get (priv->calibrate_dialog, "print-kind", &priv->print_kind, NULL);
 
@@ -830,6 +977,7 @@ gcm_calibrate_printer (GcmCalibrate *calibrate, GtkWindow *window, GError **erro
 	/* proxy */
 	ret = klass->calibrate_printer (calibrate, window, error);
 out:
+	g_free (precision);
 	return ret;
 }
 
@@ -847,6 +995,8 @@ gcm_calibrate_device (GcmCalibrate *calibrate, GtkWindow *window, GError **error
 	const gchar *directory;
 	GString *string;
 	GtkResponseType response;
+	GtkWindow *window_tmp;
+	gchar *precision = NULL;
 #ifdef GCM_USE_PACKAGEKIT
 	GtkWidget *dialog;
 #endif
@@ -897,10 +1047,10 @@ gcm_calibrate_device (GcmCalibrate *calibrate, GtkWindow *window, GError **error
 	g_string_set_size (string, 0);
 
 	/* TRANSLATORS: dialog message, preface */
-	g_string_append_printf (string, "%s\n", _("Before calibrating the device, you have to manually capture an image of a calibrated target and save it as a TIFF image file."));
+	g_string_append_printf (string, "%s\n", _("Before profiling the device, you have to manually capture an image of a calibrated target and save it as a TIFF image file."));
 
 	/* scanner specific options */
-	if (priv->device_type == GCM_DEVICE_TYPE_ENUM_SCANNER) {
+	if (priv->device_kind == GCM_DEVICE_KIND_SCANNER) {
 		/* TRANSLATORS: dialog message, preface */
 		g_string_append_printf (string, "%s\n", _("Ensure that the contrast and brightness are not changed and color correction profiles are not applied."));
 
@@ -909,7 +1059,7 @@ gcm_calibrate_device (GcmCalibrate *calibrate, GtkWindow *window, GError **error
 	}
 
 	/* camera specific options */
-	if (priv->device_type == GCM_DEVICE_TYPE_ENUM_CAMERA) {
+	if (priv->device_kind == GCM_DEVICE_KIND_CAMERA) {
 		/* TRANSLATORS: dialog message, preface */
 		g_string_append_printf (string, "%s\n", _("Ensure that the white-balance has not been modified by the camera and that the lens is clean."));
 	}
@@ -921,6 +1071,7 @@ gcm_calibrate_device (GcmCalibrate *calibrate, GtkWindow *window, GError **error
 	g_string_append_printf (string, "\n%s\n", _("Please select the chart type which corresponds to your reference file."));
 
 	/* push new messages into the UI */
+	gcm_calibrate_dialog_set_window (priv->calibrate_dialog, window);
 	gcm_calibrate_dialog_show (priv->calibrate_dialog, GCM_CALIBRATE_DIALOG_TAB_TARGET_TYPE, title, string->str);
 	gcm_calibrate_dialog_set_show_button_ok (priv->calibrate_dialog, TRUE);
 	gcm_calibrate_dialog_set_show_expander (priv->calibrate_dialog, FALSE);
@@ -938,10 +1089,21 @@ gcm_calibrate_device (GcmCalibrate *calibrate, GtkWindow *window, GError **error
 	/* copy */
 	g_object_get (priv->calibrate_dialog, "reference-kind", &priv->reference_kind, NULL);
 
+	/* get default precision */
+	precision = gconf_client_get_string (priv->gconf_client, GCM_SETTINGS_CALIBRATION_LENGTH, NULL);
+	priv->precision = gcm_calibrate_precision_from_string (precision);
+	if (priv->precision == GCM_CALIBRATE_PRECISION_UNKNOWN) {
+		priv->precision = gcm_calibrate_get_precision (calibrate, error);
+		if (priv->precision == GCM_CALIBRATE_PRECISION_UNKNOWN) {
+			ret = FALSE;
+			goto out;
+		}
+	}
+
 	/* get scanned image */
 	directory = g_get_home_dir ();
-	window = gcm_calibrate_dialog_get_window (priv->calibrate_dialog);
-	reference_image = gcm_calibrate_device_get_reference_image (directory, window);
+	window_tmp = gcm_calibrate_dialog_get_window (priv->calibrate_dialog);
+	reference_image = gcm_calibrate_device_get_reference_image (directory, window_tmp);
 	if (reference_image == NULL) {
 		g_set_error_literal (error,
 				     GCM_CALIBRATE_ERROR,
@@ -958,7 +1120,7 @@ gcm_calibrate_device (GcmCalibrate *calibrate, GtkWindow *window, GError **error
 
 	/* get reference data */
 	directory = has_shared_targets ? "/usr/share/color/targets" : "/media";
-	reference_data = gcm_calibrate_device_get_reference_data (directory, window);
+	reference_data = gcm_calibrate_device_get_reference_data (directory, window_tmp);
 	if (reference_data == NULL) {
 		g_set_error_literal (error,
 				     GCM_CALIBRATE_ERROR,
@@ -999,6 +1161,7 @@ gcm_calibrate_device (GcmCalibrate *calibrate, GtkWindow *window, GError **error
 out:
 	if (string != NULL)
 		g_string_free (string, TRUE);
+	g_free (precision);
 	g_free (device);
 	g_free (reference_image);
 	g_free (reference_data);
@@ -1024,8 +1187,8 @@ gcm_calibrate_get_property (GObject *object, guint prop_id, GValue *value, GPara
 	case PROP_PRINT_KIND:
 		g_value_set_uint (value, priv->print_kind);
 		break;
-	case PROP_DEVICE_TYPE:
-		g_value_set_uint (value, priv->device_type);
+	case PROP_CALIBRATE_DEVICE_KIND:
+		g_value_set_uint (value, priv->calibrate_device_kind);
 		break;
 	case PROP_COLORIMETER_KIND:
 		g_value_set_uint (value, priv->colorimeter_kind);
@@ -1063,6 +1226,12 @@ gcm_calibrate_get_property (GObject *object, guint prop_id, GValue *value, GPara
 	case PROP_WORKING_PATH:
 		g_value_set_string (value, priv->working_path);
 		break;
+	case PROP_PRECISION:
+		g_value_set_uint (value, priv->precision);
+		break;
+	case PROP_XYZ:
+		g_value_set_object (value, priv->xyz);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -1070,10 +1239,10 @@ gcm_calibrate_get_property (GObject *object, guint prop_id, GValue *value, GPara
 }
 
 /**
- * gcm_calibrate_guess_type:
+ * gcm_calibrate_guess_kind:
  **/
 static void
-gcm_calibrate_guess_type (GcmCalibrate *calibrate)
+gcm_calibrate_guess_kind (GcmCalibrate *calibrate)
 {
 	gboolean ret;
 	GcmCalibratePrivate *priv = calibrate->priv;
@@ -1081,7 +1250,7 @@ gcm_calibrate_guess_type (GcmCalibrate *calibrate)
 	/* guess based on the output name */
 	ret = gcm_utils_output_is_lcd_internal (priv->output_name);
 	if (ret)
-		priv->device_kind = GCM_CALIBRATE_DEVICE_KIND_LCD;
+		priv->calibrate_device_kind = GCM_CALIBRATE_DEVICE_KIND_LCD;
 }
 
 /**
@@ -1107,7 +1276,7 @@ gcm_calibrate_set_property (GObject *object, guint prop_id, const GValue *value,
 	case PROP_OUTPUT_NAME:
 		g_free (priv->output_name);
 		priv->output_name = g_strdup (g_value_get_string (value));
-		gcm_calibrate_guess_type (calibrate);
+		gcm_calibrate_guess_kind (calibrate);
 		break;
 	case PROP_FILENAME_SOURCE:
 		g_free (priv->filename_source);
@@ -1146,13 +1315,18 @@ gcm_calibrate_set_property (GObject *object, guint prop_id, const GValue *value,
 		g_free (priv->manufacturer);
 		priv->manufacturer = g_strdup (g_value_get_string (value));
 		break;
-	case PROP_DEVICE_TYPE:
-		priv->device_type = g_value_get_uint (value);
+	case PROP_DEVICE_KIND:
+		priv->device_kind = g_value_get_uint (value);
 		break;
 	case PROP_WORKING_PATH:
 		g_free (priv->working_path);
 		priv->working_path = g_strdup (g_value_get_string (value));
 		break;
+	case PROP_XYZ:
+		if (priv->xyz != NULL)
+			g_object_unref (priv->xyz);
+		priv->xyz = g_value_dup_object (value);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -1180,12 +1354,12 @@ gcm_calibrate_class_init (GcmCalibrateClass *klass)
 	g_object_class_install_property (object_class, PROP_REFERENCE_KIND, pspec);
 
 	/**
-	 * GcmCalibrate:device-kind:
+	 * GcmCalibrate:calibrate-device-kind:
 	 */
-	pspec = g_param_spec_uint ("device-kind", NULL, NULL,
+	pspec = g_param_spec_uint ("calibrate-device-kind", NULL, NULL,
 				   0, G_MAXUINT, 0,
 				   G_PARAM_READABLE);
-	g_object_class_install_property (object_class, PROP_DEVICE_KIND, pspec);
+	g_object_class_install_property (object_class, PROP_CALIBRATE_DEVICE_KIND, pspec);
 
 	/**
 	 * GcmCalibrate:print-kind:
@@ -1196,12 +1370,12 @@ gcm_calibrate_class_init (GcmCalibrateClass *klass)
 	g_object_class_install_property (object_class, PROP_PRINT_KIND, pspec);
 
 	/**
-	 * GcmCalibrate:device-type:
+	 * GcmCalibrate:device-kind:
 	 */
-	pspec = g_param_spec_uint ("device-type", NULL, NULL,
+	pspec = g_param_spec_uint ("device-kind", NULL, NULL,
 				   0, G_MAXUINT, 0,
 				   G_PARAM_READWRITE);
-	g_object_class_install_property (object_class, PROP_DEVICE_TYPE, pspec);
+	g_object_class_install_property (object_class, PROP_DEVICE_KIND, pspec);
 
 	/**
 	 * GcmCalibrate:colorimeter-kind:
@@ -1299,6 +1473,22 @@ gcm_calibrate_class_init (GcmCalibrateClass *klass)
 				     G_PARAM_READWRITE);
 	g_object_class_install_property (object_class, PROP_WORKING_PATH, pspec);
 
+	/**
+	 * GcmCalibrate:precision:
+	 */
+	pspec = g_param_spec_uint ("precision", NULL, NULL,
+				   0, G_MAXUINT, 0,
+				   G_PARAM_READABLE);
+	g_object_class_install_property (object_class, PROP_PRECISION, pspec);
+
+	/**
+	 * GcmCalibrate:xyz:
+	 */
+	pspec = g_param_spec_object ("xyz", NULL, NULL,
+				     GCM_TYPE_XYZ,
+				     G_PARAM_READWRITE);
+	g_object_class_install_property (object_class, PROP_XYZ, pspec);
+
 	g_type_class_add_private (klass, sizeof (GcmCalibratePrivate));
 }
 
@@ -1319,16 +1509,22 @@ gcm_calibrate_init (GcmCalibrate *calibrate)
 	calibrate->priv->description = NULL;
 	calibrate->priv->device = NULL;
 	calibrate->priv->serial = NULL;
+	calibrate->priv->xyz = gcm_xyz_new ();
 	calibrate->priv->working_path = NULL;
-	calibrate->priv->device_kind = GCM_CALIBRATE_DEVICE_KIND_UNKNOWN;
+	calibrate->priv->calibrate_device_kind = GCM_CALIBRATE_DEVICE_KIND_UNKNOWN;
 	calibrate->priv->print_kind = GCM_CALIBRATE_PRINT_KIND_UNKNOWN;
 	calibrate->priv->reference_kind = GCM_CALIBRATE_REFERENCE_KIND_UNKNOWN;
+	calibrate->priv->precision = GCM_CALIBRATE_PRECISION_UNKNOWN;
 	calibrate->priv->colorimeter = gcm_colorimeter_new ();
+	calibrate->priv->dmi = gcm_dmi_new ();
 	calibrate->priv->calibrate_dialog = gcm_calibrate_dialog_new ();
 
 	// FIXME: this has to be per-run specific
 	calibrate->priv->working_path = g_strdup ("/tmp");
 
+	/* use GConf to get defaults */
+	calibrate->priv->gconf_client = gconf_client_get_default ();
+
 	/* coldplug, and watch for changes */
 	calibrate->priv->colorimeter_kind = gcm_colorimeter_get_kind (calibrate->priv->colorimeter);
 	g_signal_connect (calibrate->priv->colorimeter, "changed", G_CALLBACK (gcm_prefs_colorimeter_changed_cb), calibrate);
@@ -1355,8 +1551,12 @@ gcm_calibrate_finalize (GObject *object)
 	g_free (priv->serial);
 	g_free (priv->working_path);
 	g_signal_handlers_disconnect_by_func (calibrate->priv->colorimeter, G_CALLBACK (gcm_prefs_colorimeter_changed_cb), calibrate);
+	if (priv->xyz != NULL)
+		g_object_unref (priv->xyz);
 	g_object_unref (priv->colorimeter);
+	g_object_unref (priv->dmi);
 	g_object_unref (priv->calibrate_dialog);
+	g_object_unref (priv->gconf_client);
 
 	G_OBJECT_CLASS (gcm_calibrate_parent_class)->finalize (object);
 }
