@@ -51,6 +51,7 @@ static void     gcm_device_xrandr_finalize	(GObject     *object);
 struct _GcmDeviceXrandrPrivate
 {
 	gchar				*native_device;
+	gchar				*eisa_id;
 	guint				 gamma_size;
 	GcmEdid				*edid;
 	GcmDmi				*dmi;
@@ -64,6 +65,7 @@ enum {
 	PROP_0,
 	PROP_NATIVE_DEVICE,
 	PROP_XRANDR_FALLBACK,
+	PROP_EISA_ID,
 	PROP_LAST
 };
 
@@ -72,6 +74,33 @@ G_DEFINE_TYPE (GcmDeviceXrandr, gcm_device_xrandr, GCM_TYPE_DEVICE)
 #define GCM_ICC_PROFILE_IN_X_VERSION_MAJOR	0
 #define GCM_ICC_PROFILE_IN_X_VERSION_MINOR	3
 
+/**
+ * gcm_device_xrandr_get_native_device:
+ **/
+const gchar *
+gcm_device_xrandr_get_native_device (GcmDeviceXrandr *device_xrandr)
+{
+	return device_xrandr->priv->native_device;
+}
+
+/**
+ * gcm_device_xrandr_get_eisa_id:
+ **/
+const gchar *
+gcm_device_xrandr_get_eisa_id (GcmDeviceXrandr *device_xrandr)
+{
+	return device_xrandr->priv->eisa_id;
+}
+
+/**
+ * gcm_device_xrandr_get_fallback:
+ **/
+gboolean
+gcm_device_xrandr_get_fallback (GcmDeviceXrandr *device_xrandr)
+{
+	return device_xrandr->priv->xrandr_fallback;
+}
+
 /**
  * gcm_device_xrandr_get_output_name:
  *
@@ -81,8 +110,8 @@ static gchar *
 gcm_device_xrandr_get_output_name (GcmDeviceXrandr *device_xrandr, GnomeRROutput *output)
 {
 	const gchar *output_name;
-	gchar *name = NULL;
-	gchar *vendor = NULL;
+	const gchar *name;
+	const gchar *vendor;
 	GString *string;
 	gboolean ret;
 	guint width = 0;
@@ -102,19 +131,15 @@ gcm_device_xrandr_get_output_name (GcmDeviceXrandr *device_xrandr, GnomeRROutput
 	ret = gcm_utils_output_is_lcd_internal (output_name);
 	if (ret) {
 		/* find the machine details */
-		g_object_get (priv->dmi,
-			      "name", &name,
-			      "vendor", &vendor,
-			      NULL);
+		name = gcm_dmi_get_name (priv->dmi);
+		vendor = gcm_dmi_get_vendor (priv->dmi);
 
 		/* TRANSLATORS: this is the name of the internal panel */
 		output_name = _("Laptop LCD");
 	} else {
 		/* find the panel details */
-		g_object_get (priv->edid,
-			      "monitor-name", &name,
-			      "vendor-name", &vendor,
-			      NULL);
+		name = gcm_edid_get_monitor_name (priv->edid);
+		vendor = gcm_edid_get_vendor_name (priv->edid);
 	}
 
 	/* prepend vendor if available */
@@ -130,13 +155,9 @@ gcm_device_xrandr_get_output_name (GcmDeviceXrandr *device_xrandr, GnomeRROutput
 		g_string_assign (string, "Unknown Monitor");
 
 out:
-	/* find the best option */
-	g_object_get (priv->edid,
-		      "width", &width,
-		      "height", &height,
-		      NULL);
-
 	/* append size if available */
+	width = gcm_edid_get_width (priv->edid);
+	height = gcm_edid_get_height (priv->edid);
 	if (width != 0 && height != 0) {
 		gfloat diag;
 
@@ -147,8 +168,6 @@ out:
 		g_string_append_printf (string, " - %i\"", (guint) ((diag * 0.393700787f) + 0.5f));
 	}
 
-	g_free (name);
-	g_free (vendor);
 	return g_string_free (string, FALSE);
 }
 
@@ -159,10 +178,10 @@ static gchar *
 gcm_device_xrandr_get_id_for_xrandr_device (GcmDeviceXrandr *device_xrandr, GnomeRROutput *output)
 {
 	const gchar *output_name;
-	gchar *name = NULL;
-	gchar *vendor = NULL;
-	gchar *ascii = NULL;
-	gchar *serial = NULL;
+	const gchar *name;
+	const gchar *vendor;
+	const gchar *ascii;
+	const gchar *serial;
 	GString *string;
 	gboolean ret;
 	GcmDeviceXrandrPrivate *priv = device_xrandr->priv;
@@ -175,21 +194,17 @@ gcm_device_xrandr_get_id_for_xrandr_device (GcmDeviceXrandr *device_xrandr, Gnom
 	if (!ret)
 		goto out;
 
-	/* find the best option */
-	g_object_get (priv->edid,
-		      "monitor-name", &name,
-		      "vendor-name", &vendor,
-		      "ascii-string", &ascii,
-		      "serial-number", &serial,
-		      NULL);
-
 	/* append data if available */
+	vendor = gcm_edid_get_vendor_name (priv->edid);
 	if (vendor != NULL)
 		g_string_append_printf (string, "_%s", vendor);
+	name = gcm_edid_get_monitor_name (priv->edid);
 	if (name != NULL)
 		g_string_append_printf (string, "_%s", name);
+	ascii = gcm_edid_get_eisa_id (priv->edid);
 	if (ascii != NULL)
 		g_string_append_printf (string, "_%s", ascii);
+	serial = gcm_edid_get_serial_number (priv->edid);
 	if (serial != NULL)
 		g_string_append_printf (string, "_%s", serial);
 out:
@@ -204,11 +219,6 @@ out:
 
 	/* replace unsafe chars */
 	gcm_utils_alphanum_lcase (string->str);
-
-	g_free (name);
-	g_free (vendor);
-	g_free (ascii);
-	g_free (serial);
 	return g_string_free (string, FALSE);
 }
 
@@ -223,9 +233,9 @@ gcm_device_xrandr_set_from_output (GcmDevice *device, GnomeRROutput *output, GEr
 	gboolean ret = TRUE;
 	gboolean lcd_internal;
 	const gchar *output_name;
-	gchar *serial = NULL;
-	gchar *manufacturer = NULL;
-	gchar *model = NULL;
+	const gchar *serial;
+	const gchar *manufacturer;
+	const gchar *model;
 	const guint8 *data;
 	GcmDeviceXrandrPrivate *priv = GCM_DEVICE_XRANDR(device)->priv;
 
@@ -247,26 +257,22 @@ gcm_device_xrandr_set_from_output (GcmDevice *device, GnomeRROutput *output, GEr
 	egg_debug ("asking to add %s", id);
 
 	/* get data about the display */
-	g_object_get (priv->edid,
-		      "monitor-name", &model,
-		      "vendor-name", &manufacturer,
-		      "serial-number", &serial,
-		      NULL);
+	model = gcm_edid_get_monitor_name (priv->edid);
+	manufacturer = gcm_edid_get_vendor_name (priv->edid);
+	serial = gcm_edid_get_serial_number (priv->edid);
+	priv->eisa_id = g_strdup (gcm_edid_get_eisa_id (priv->edid));
 
 	/* refine data if it's missing */
 	output_name = gnome_rr_output_get_name (output);
 	lcd_internal = gcm_utils_output_is_lcd_internal (output_name);
-	if (lcd_internal && model == NULL) {
-		g_object_get (priv->dmi,
-			      "version", &model,
-			      NULL);
-	}
+	if (lcd_internal && model == NULL)
+		model = gcm_dmi_get_version (priv->dmi);
 
 	/* add new device */
 	title = gcm_device_xrandr_get_output_name (GCM_DEVICE_XRANDR(device), output);
 	g_object_set (device,
-		      "type", GCM_DEVICE_TYPE_ENUM_DISPLAY,
-		      "colorspace", GCM_COLORSPACE_ENUM_RGB,
+		      "kind", GCM_DEVICE_KIND_DISPLAY,
+		      "colorspace", GCM_COLORSPACE_RGB,
 		      "id", id,
 		      "connected", TRUE,
 		      "serial", serial,
@@ -277,9 +283,6 @@ gcm_device_xrandr_set_from_output (GcmDevice *device, GnomeRROutput *output, GEr
 		      NULL);
 out:
 	g_free (id);
-	g_free (serial);
-	g_free (manufacturer);
-	g_free (model);
 	g_free (title);
 	return ret;
 }
@@ -321,7 +324,7 @@ gcm_device_xrandr_get_gamma_size (GcmDeviceXrandr *device_xrandr, GnomeRRCrtc *c
 	if (priv->gamma_size > 0)
 		return priv->gamma_size;
 
-	/* get id that X recognises */
+	/* get id that X recognizes */
 	id = gnome_rr_crtc_get_id (crtc);
 
 	/* get the value, and catch errors */
@@ -406,7 +409,7 @@ gcm_device_xrandr_apply_for_crtc (GcmDeviceXrandr *device_xrandr, GnomeRRCrtc *c
 		crtc_gamma->blue[i] = data->blue;
 	}
 
-	/* get id that X recognises */
+	/* get id that X recognizes */
 	id = gnome_rr_crtc_get_id (crtc);
 
 	/* get the value, and catch errors */
@@ -443,48 +446,41 @@ gcm_device_xrandr_apply (GcmDevice *device, GError **error)
 	GnomeRRCrtc *crtc;
 	GnomeRROutput *output;
 	gint x, y;
-	gchar *filename = NULL;
+	const gchar *filename;
 	gchar *filename_systemwide = NULL;
 	gfloat gamma_adjust;
 	gfloat brightness;
 	gfloat contrast;
-	gchar *output_name;
-	gchar *id = NULL;
+	const gchar *output_name;
+	const gchar *id;
 	guint size;
 	gboolean saved;
 	gboolean use_global;
 	gboolean use_atom;
 	gboolean leftmost_screen = FALSE;
 	GFile *file = NULL;
-	GcmDeviceTypeEnum type;
+	GcmDeviceKind kind;
 	GcmDeviceXrandr *device_xrandr = GCM_DEVICE_XRANDR (device);
 	GcmDeviceXrandrPrivate *priv = device_xrandr->priv;
 
-	/* get details about the device */
-	g_object_get (device_xrandr,
-		      "id", &id,
-		      "type", &type,
-		      "saved", &saved,
-		      "profile-filename", &filename,
-		      "gamma", &gamma_adjust,
-		      "brightness", &brightness,
-		      "contrast", &contrast,
-		      "native-device", &output_name,
-		      NULL);
-
 	/* do no set the gamma for non-display types */
-	if (type != GCM_DEVICE_TYPE_ENUM_DISPLAY) {
+	id = gcm_device_get_id (device);
+	kind = gcm_device_get_kind (device);
+	if (kind != GCM_DEVICE_KIND_DISPLAY) {
 		g_set_error (error, 1, 0, "not a display: %s", id);
 		goto out;
 	}
 
 	/* should be set for display types */
+	output_name = gcm_device_xrandr_get_native_device (device_xrandr);
 	if (output_name == NULL || output_name[0] == '\0') {
 		g_set_error (error, 1, 0, "no output name for display: %s", id);
 		goto out;
 	}
 
 	/* if not saved, try to find default filename */
+	saved = gcm_device_get_saved (device);
+	filename = gcm_device_get_profile_filename (device);
 	if (!saved && filename == NULL) {
 		filename_systemwide = g_strdup_printf ("%s/%s.icc", GCM_SYSTEM_PROFILES_DIR, id);
 		ret = g_file_test (filename_systemwide, G_FILE_TEST_EXISTS);
@@ -536,6 +532,9 @@ gcm_device_xrandr_apply (GcmDevice *device, GError **error)
 
 	/* do fine adjustment */
 	if (use_global) {
+		gamma_adjust = gcm_device_get_gamma (device);
+		brightness = gcm_device_get_brightness (device);
+		contrast = gcm_device_get_contrast (device);
 		g_object_set (clut,
 			      "gamma", gamma_adjust,
 			      "brightness", brightness,
@@ -590,10 +589,7 @@ gcm_device_xrandr_apply (GcmDevice *device, GError **error)
 		}
 	}
 out:
-	g_free (id);
-	g_free (filename);
 	g_free (filename_systemwide);
-	g_free (output_name);
 	if (clut != NULL)
 		g_object_unref (clut);
 	if (profile != NULL)
@@ -617,6 +613,9 @@ gcm_device_xrandr_get_property (GObject *object, guint prop_id, GValue *value, G
 	case PROP_XRANDR_FALLBACK:
 		g_value_set_boolean (value, priv->xrandr_fallback);
 		break;
+	case PROP_EISA_ID:
+		g_value_set_string (value, priv->eisa_id);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -675,6 +674,14 @@ gcm_device_xrandr_class_init (GcmDeviceXrandrClass *klass)
 				      G_PARAM_READABLE);
 	g_object_class_install_property (object_class, PROP_XRANDR_FALLBACK, pspec);
 
+	/**
+	 * GcmDeviceXrandr:eisa-id:
+	 */
+	pspec = g_param_spec_string ("eisa-id", NULL, NULL,
+				     NULL,
+				     G_PARAM_READABLE);
+	g_object_class_install_property (object_class, PROP_EISA_ID, pspec);
+
 	g_type_class_add_private (klass, sizeof (GcmDeviceXrandrPrivate));
 }
 
@@ -686,6 +693,7 @@ gcm_device_xrandr_init (GcmDeviceXrandr *device_xrandr)
 {
 	device_xrandr->priv = GCM_DEVICE_XRANDR_GET_PRIVATE (device_xrandr);
 	device_xrandr->priv->native_device = NULL;
+	device_xrandr->priv->eisa_id = NULL;
 	device_xrandr->priv->xrandr_fallback = FALSE;
 	device_xrandr->priv->gamma_size = 0;
 	device_xrandr->priv->edid = gcm_edid_new ();
@@ -705,6 +713,7 @@ gcm_device_xrandr_finalize (GObject *object)
 	GcmDeviceXrandrPrivate *priv = device_xrandr->priv;
 
 	g_free (priv->native_device);
+	g_free (priv->eisa_id);
 	g_object_unref (priv->edid);
 	g_object_unref (priv->dmi);
 	g_object_unref (priv->gconf_client);
