@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2009 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2009-2010 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -53,7 +53,7 @@ struct _GcmEdidPrivate
 	gchar				*monitor_name;
 	gchar				*vendor_name;
 	gchar				*serial_number;
-	gchar				*ascii_string;
+	gchar				*eisa_id;
 	gchar				*pnp_id;
 	guint				 width;
 	guint				 height;
@@ -66,7 +66,7 @@ enum {
 	PROP_MONITOR_NAME,
 	PROP_VENDOR_NAME,
 	PROP_SERIAL_NUMBER,
-	PROP_ASCII_STRING,
+	PROP_EISA_ID,
 	PROP_GAMMA,
 	PROP_PNP_ID,
 	PROP_WIDTH,
@@ -91,6 +91,90 @@ G_DEFINE_TYPE (GcmEdid, gcm_edid, G_TYPE_OBJECT)
 #define GCM_DESCRIPTOR_COLOR_POINT			0xfb
 
 
+/**
+ * gcm_edid_get_monitor_name:
+ **/
+const gchar *
+gcm_edid_get_monitor_name (GcmEdid *edid)
+{
+	g_return_val_if_fail (GCM_IS_EDID (edid), NULL);
+	return edid->priv->monitor_name;
+}
+
+/**
+ * gcm_edid_get_vendor_name:
+ **/
+const gchar *
+gcm_edid_get_vendor_name (GcmEdid *edid)
+{
+	GcmEdidPrivate *priv = edid->priv;
+	g_return_val_if_fail (GCM_IS_EDID (edid), NULL);
+
+	if (priv->vendor_name == NULL)
+		priv->vendor_name = gcm_tables_get_pnp_id (priv->tables, priv->pnp_id, NULL);
+	return priv->vendor_name;
+}
+
+/**
+ * gcm_edid_get_serial_number:
+ **/
+const gchar *
+gcm_edid_get_serial_number (GcmEdid *edid)
+{
+	g_return_val_if_fail (GCM_IS_EDID (edid), NULL);
+	return edid->priv->serial_number;
+}
+
+/**
+ * gcm_edid_get_eisa_id:
+ **/
+const gchar *
+gcm_edid_get_eisa_id (GcmEdid *edid)
+{
+	g_return_val_if_fail (GCM_IS_EDID (edid), NULL);
+	return edid->priv->eisa_id;
+}
+
+/**
+ * gcm_edid_get_pnp_id:
+ **/
+const gchar *
+gcm_edid_get_pnp_id (GcmEdid *edid)
+{
+	g_return_val_if_fail (GCM_IS_EDID (edid), NULL);
+	return edid->priv->pnp_id;
+}
+
+/**
+ * gcm_edid_get_width:
+ **/
+guint
+gcm_edid_get_width (GcmEdid *edid)
+{
+	g_return_val_if_fail (GCM_IS_EDID (edid), 0);
+	return edid->priv->width;
+}
+
+/**
+ * gcm_edid_get_height:
+ **/
+guint
+gcm_edid_get_height (GcmEdid *edid)
+{
+	g_return_val_if_fail (GCM_IS_EDID (edid), 0);
+	return edid->priv->height;
+}
+
+/**
+ * gcm_edid_get_gamma:
+ **/
+gfloat
+gcm_edid_get_gamma (GcmEdid *edid)
+{
+	g_return_val_if_fail (GCM_IS_EDID (edid), 0.0f);
+	return edid->priv->gamma;
+}
+
 /**
  * gcm_edid_reset:
  **/
@@ -105,7 +189,7 @@ gcm_edid_reset (GcmEdid *edid)
 	g_free (priv->monitor_name);
 	g_free (priv->vendor_name);
 	g_free (priv->serial_number);
-	g_free (priv->ascii_string);
+	g_free (priv->eisa_id);
 
 	/* do not deallocate, just blank */
 	priv->pnp_id[0] = '\0';
@@ -114,12 +198,79 @@ gcm_edid_reset (GcmEdid *edid)
 	priv->monitor_name = NULL;
 	priv->vendor_name = NULL;
 	priv->serial_number = NULL;
-	priv->ascii_string = NULL;
+	priv->eisa_id = NULL;
 	priv->width = 0;
 	priv->height = 0;
 	priv->gamma = 0.0f;
 }
 
+/**
+ * gcm_edid_get_bit:
+ *
+ * Originally Copyright Soren Sandmann <sandmann@redhat.com>
+ **/
+static gint
+gcm_edid_get_bit (gint in, gint bit)
+{
+	return (in & (1 << bit)) >> bit;
+}
+
+/**
+ * gcm_edid_get_bits:
+ **/
+static gint
+gcm_edid_get_bits (gint in, gint begin, gint end)
+{
+	gint mask = (1 << (end - begin + 1)) - 1;
+
+	return (in >> begin) & mask;
+}
+
+/**
+ * gcm_edid_decode_fraction:
+ **/
+static gdouble
+gcm_edid_decode_fraction (gint high, gint low)
+{
+	gdouble result = 0.0;
+	gint i;
+
+	high = (high << 2) | low;
+	for (i = 0; i < 10; ++i)
+		result += gcm_edid_get_bit (high, i) * pow (2, i - 10);
+	return result;
+}
+
+/**
+ * gcm_edid_parse_string:
+ *
+ * This is always 12 bytes, but we can't guarantee it's null terminated
+ * or not junk.
+ *
+ * Return value: the sane text, or %NULL, use g_free() to unref.
+ **/
+static gchar *
+gcm_edid_parse_string (const guint8 *data)
+{
+	gchar *text;
+
+	/* copy 12 bytes */
+	text = g_strndup ((const gchar *) data, 12);
+
+	/* remove insane newline chars */
+	g_strdelimit (text, "\n\r", '\0');
+
+	/* remove spaces */
+	g_strchomp (text);
+
+	/* nothing left? */
+	if (text[0] == '\0') {
+		g_free (text);
+		text = NULL;
+	}
+	return text;
+}
+
 /**
  * gcm_edid_parse:
  **/
@@ -131,6 +282,8 @@ gcm_edid_parse (GcmEdid *edid, const guint8 *data, GError **error)
 	GcmEdidPrivate *priv = edid->priv;
 	guint32 serial;
 	guint extension_blocks;
+	gdouble x, y;
+	gchar *tmp;
 
 	g_return_val_if_fail (GCM_IS_EDID (edid), FALSE);
 	g_return_val_if_fail (data != NULL, FALSE);
@@ -184,6 +337,26 @@ gcm_edid_parse (GcmEdid *edid, const guint8 *data, GError **error)
 		egg_debug ("gamma is reported as %f", priv->gamma);
 	}
 
+	/* get color red */
+	x = gcm_edid_decode_fraction (data[0x1b], gcm_edid_get_bits (data[0x19], 6, 7));
+	y = gcm_edid_decode_fraction (data[0x1c], gcm_edid_get_bits (data[0x19], 5, 4));
+	egg_debug ("red x=%f,y=%f", x, y);
+
+	/* get color green */
+	x = gcm_edid_decode_fraction (data[0x1d], gcm_edid_get_bits (data[0x19], 2, 3));
+	y = gcm_edid_decode_fraction (data[0x1e], gcm_edid_get_bits (data[0x19], 0, 1));
+	egg_debug ("green x=%f,y=%f", x, y);
+
+	/* get color blue */
+	x = gcm_edid_decode_fraction (data[0x1f], gcm_edid_get_bits (data[0x1a], 6, 7));
+	y = gcm_edid_decode_fraction (data[0x20], gcm_edid_get_bits (data[0x1a], 4, 5));
+	egg_debug ("blue x=%f,y=%f", x, y);
+
+	/* get color white */
+	x = gcm_edid_decode_fraction (data[0x21], gcm_edid_get_bits (data[0x1a], 2, 3));
+	y = gcm_edid_decode_fraction (data[0x22], gcm_edid_get_bits (data[0x1a], 0, 1));
+	egg_debug ("white x=%f,y=%f", x, y);
+
 	/* parse EDID data */
 	for (i=GCM_EDID_OFFSET_DATA_BLOCKS; i <= GCM_EDID_OFFSET_LAST_BLOCK; i+=18) {
 		/* ignore pixel clock data */
@@ -194,17 +367,25 @@ gcm_edid_parse (GcmEdid *edid, const guint8 *data, GError **error)
 
 		/* any useful blocks? */
 		if (data[i+3] == GCM_DESCRIPTOR_DISPLAY_PRODUCT_NAME) {
-			priv->monitor_name = g_strndup ((const gchar *) &data[i+5], 12);
-			egg_debug ("[extended EDID block] monitor name: %s", priv->monitor_name);
+			tmp = gcm_edid_parse_string (&data[i+5]);
+			if (tmp != NULL) {
+				g_free (priv->monitor_name);
+				priv->monitor_name = tmp;
+			}
 		} else if (data[i+3] == GCM_DESCRIPTOR_DISPLAY_PRODUCT_SERIAL_NUMBER) {
-			g_free (priv->serial_number);
-			priv->serial_number = g_strndup ((const gchar *) &data[i+5], 12);
-			egg_debug ("[extended EDID block] serial number: %s", priv->serial_number);
+			tmp = gcm_edid_parse_string (&data[i+5]);
+			if (tmp != NULL) {
+				g_free (priv->serial_number);
+				priv->serial_number = tmp;
+			}
 		} else if (data[i+3] == GCM_DESCRIPTOR_COLOR_MANAGEMENT_DATA) {
 			egg_warning ("failing to parse color management data");
 		} else if (data[i+3] == GCM_DESCRIPTOR_ALPHANUMERIC_DATA_STRING) {
-			priv->ascii_string = g_strndup ((const gchar *) &data[i+5], 12);
-			egg_debug ("[extended EDID block] ascii string: %s", priv->ascii_string);
+			tmp = gcm_edid_parse_string (&data[i+5]);
+			if (tmp != NULL) {
+				g_free (priv->eisa_id);
+				priv->eisa_id = tmp;
+			}
 		} else if (data[i+3] == GCM_DESCRIPTOR_COLOR_POINT) {
 			if (data[i+3+9] != 0xff) {
 				egg_debug ("extended EDID block(1) which contains a better gamma value");
@@ -224,19 +405,10 @@ gcm_edid_parse (GcmEdid *edid, const guint8 *data, GError **error)
 	if (extension_blocks > 0)
 		egg_warning ("%i extension blocks to parse", extension_blocks);
 
-	/* remove embedded newlines */
-	if (priv->monitor_name != NULL) {
-		g_strchomp (priv->monitor_name);
-		g_strdelimit (priv->monitor_name, "\n\r", '\0');
-	}
-	if (priv->serial_number != NULL) {
-		g_strchomp (priv->serial_number);
-		g_strdelimit (priv->serial_number, "\n\r", '\0');
-	}
-	if (priv->ascii_string != NULL) {
-		g_strchomp (priv->ascii_string);
-		g_strdelimit (priv->ascii_string, "\n\r", '\0');
-	}
+	/* print what we've got */
+	egg_debug ("monitor name: %s", priv->monitor_name);
+	egg_debug ("serial number: %s", priv->serial_number);
+	egg_debug ("ascii string: %s", priv->eisa_id);
 out:
 	return ret;
 }
@@ -255,15 +427,13 @@ gcm_edid_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec
 		g_value_set_string (value, priv->monitor_name);
 		break;
 	case PROP_VENDOR_NAME:
-		if (priv->vendor_name == NULL)
-			priv->vendor_name = gcm_tables_get_pnp_id (priv->tables, priv->pnp_id, NULL);
-		g_value_set_string (value, priv->vendor_name);
+		g_value_set_string (value, gcm_edid_get_vendor_name (edid));
 		break;
 	case PROP_SERIAL_NUMBER:
 		g_value_set_string (value, priv->serial_number);
 		break;
-	case PROP_ASCII_STRING:
-		g_value_set_string (value, priv->ascii_string);
+	case PROP_EISA_ID:
+		g_value_set_string (value, priv->eisa_id);
 		break;
 	case PROP_GAMMA:
 		g_value_set_float (value, priv->gamma);
@@ -333,12 +503,12 @@ gcm_edid_class_init (GcmEdidClass *klass)
 	g_object_class_install_property (object_class, PROP_SERIAL_NUMBER, pspec);
 
 	/**
-	 * GcmEdid:ascii-string:
+	 * GcmEdid:eisa-id:
 	 */
-	pspec = g_param_spec_string ("ascii-string", NULL, NULL,
+	pspec = g_param_spec_string ("eisa-id", NULL, NULL,
 				     NULL,
 				     G_PARAM_READABLE);
-	g_object_class_install_property (object_class, PROP_ASCII_STRING, pspec);
+	g_object_class_install_property (object_class, PROP_EISA_ID, pspec);
 
 	/**
 	 * GcmEdid:gamma:
@@ -385,7 +555,7 @@ gcm_edid_init (GcmEdid *edid)
 	edid->priv->monitor_name = NULL;
 	edid->priv->vendor_name = NULL;
 	edid->priv->serial_number = NULL;
-	edid->priv->ascii_string = NULL;
+	edid->priv->eisa_id = NULL;
 	edid->priv->tables = gcm_tables_new ();
 	edid->priv->pnp_id = g_new0 (gchar, 4);
 }
@@ -402,7 +572,7 @@ gcm_edid_finalize (GObject *object)
 	g_free (priv->monitor_name);
 	g_free (priv->vendor_name);
 	g_free (priv->serial_number);
-	g_free (priv->ascii_string);
+	g_free (priv->eisa_id);
 	g_free (priv->pnp_id);
 	g_object_unref (priv->tables);
 
@@ -432,7 +602,7 @@ typedef struct {
 	const gchar *monitor_name;
 	const gchar *vendor_name;
 	const gchar *serial_number;
-	const gchar *ascii_string;
+	const gchar *eisa_id;
 	const gchar *pnp_id;
 	guint width;
 	guint height;
@@ -443,11 +613,11 @@ void
 gcm_edid_test_parse_edid_file (EggTest *test, GcmEdid *edid, const gchar *datafile, GcmEdidTestData *test_data)
 {
 	gchar *filename;
-	gchar *monitor_name;
-	gchar *vendor_name;
-	gchar *serial_number;
-	gchar *ascii_string;
-	gchar *pnp_id;
+	const gchar *monitor_name;
+	const gchar *vendor_name;
+	const gchar *serial_number;
+	const gchar *eisa_id;
+	const gchar *pnp_id;
 	gchar *data;
 	guint width;
 	guint height;
@@ -472,19 +642,9 @@ gcm_edid_test_parse_edid_file (EggTest *test, GcmEdid *edid, const gchar *datafi
 	else
 		egg_test_failed (test, "failed to parse: %s", error->message);
 
-	g_object_get (edid,
-		      "monitor-name", &monitor_name,
-		      "vendor-name", &vendor_name,
-		      "serial-number", &serial_number,
-		      "ascii-string", &ascii_string,
-		      "pnp-id", &pnp_id,
-		      "width", &width,
-		      "height", &height,
-		      "gamma", &gamma,
-		      NULL);
-
 	/************************************************************/
 	egg_test_title (test, "check monitor name for %s", datafile);
+	monitor_name = gcm_edid_get_monitor_name (edid);
 	if (g_strcmp0 (monitor_name, test_data->monitor_name) == 0)
 		egg_test_success (test, NULL);
 	else
@@ -492,6 +652,7 @@ gcm_edid_test_parse_edid_file (EggTest *test, GcmEdid *edid, const gchar *datafi
 
 	/************************************************************/
 	egg_test_title (test, "check vendor name for %s", datafile);
+	vendor_name = gcm_edid_get_vendor_name (edid);
 	if (g_strcmp0 (vendor_name, test_data->vendor_name) == 0)
 		egg_test_success (test, NULL);
 	else
@@ -499,6 +660,7 @@ gcm_edid_test_parse_edid_file (EggTest *test, GcmEdid *edid, const gchar *datafi
 
 	/************************************************************/
 	egg_test_title (test, "check serial number for %s", datafile);
+	serial_number = gcm_edid_get_serial_number (edid);
 	if (g_strcmp0 (serial_number, test_data->serial_number) == 0)
 		egg_test_success (test, NULL);
 	else
@@ -506,13 +668,15 @@ gcm_edid_test_parse_edid_file (EggTest *test, GcmEdid *edid, const gchar *datafi
 
 	/************************************************************/
 	egg_test_title (test, "check ascii string for %s", datafile);
-	if (g_strcmp0 (ascii_string, test_data->ascii_string) == 0)
+	eisa_id = gcm_edid_get_eisa_id (edid);
+	if (g_strcmp0 (eisa_id, test_data->eisa_id) == 0)
 		egg_test_success (test, NULL);
 	else
-		egg_test_failed (test, "invalid value: %s, expecting: %s", ascii_string, test_data->ascii_string);
+		egg_test_failed (test, "invalid value: %s, expecting: %s", eisa_id, test_data->eisa_id);
 
 	/************************************************************/
 	egg_test_title (test, "check pnp id for %s", datafile);
+	pnp_id = gcm_edid_get_pnp_id (edid);
 	if (g_strcmp0 (pnp_id, test_data->pnp_id) == 0)
 		egg_test_success (test, NULL);
 	else
@@ -520,6 +684,7 @@ gcm_edid_test_parse_edid_file (EggTest *test, GcmEdid *edid, const gchar *datafi
 
 	/************************************************************/
 	egg_test_title (test, "check height for %s", datafile);
+	height = gcm_edid_get_height (edid);
 	if (height == test_data->height)
 		egg_test_success (test, NULL);
 	else
@@ -527,6 +692,7 @@ gcm_edid_test_parse_edid_file (EggTest *test, GcmEdid *edid, const gchar *datafi
 
 	/************************************************************/
 	egg_test_title (test, "check width for %s", datafile);
+	width = gcm_edid_get_width (edid);
 	if (width == test_data->width)
 		egg_test_success (test, NULL);
 	else
@@ -534,17 +700,12 @@ gcm_edid_test_parse_edid_file (EggTest *test, GcmEdid *edid, const gchar *datafi
 
 	/************************************************************/
 	egg_test_title (test, "check gamma for %s", datafile);
+	gamma = gcm_edid_get_gamma (edid);
 	if (gamma > (test_data->gamma - 0.01) && gamma < (test_data->gamma + 0.01))
 		egg_test_success (test, NULL);
 	else
 		egg_test_failed (test, "invalid value: %f, expecting: %f", gamma, test_data->gamma);
 
-	g_free (monitor_name);
-	g_free (vendor_name);
-	g_free (serial_number);
-	g_free (ascii_string);
-	g_free (pnp_id);
-	g_free (data);
 	g_free (filename);
 }
 
@@ -568,7 +729,7 @@ gcm_edid_test (EggTest *test)
 	test_data.monitor_name = "L225W";
 	test_data.vendor_name = "Goldstar Company Ltd";
 	test_data.serial_number = "34398";
-	test_data.ascii_string = NULL;
+	test_data.eisa_id = NULL;
 	test_data.pnp_id = "GSM";
 	test_data.height = 30;
 	test_data.width = 47;
@@ -579,7 +740,7 @@ gcm_edid_test (EggTest *test)
 	test_data.monitor_name = NULL;
 	test_data.vendor_name = "IBM France";
 	test_data.serial_number = NULL;
-	test_data.ascii_string = "LTN154P2-L05";
+	test_data.eisa_id = "LTN154P2-L05";
 	test_data.pnp_id = "IBM";
 	test_data.height = 21;
 	test_data.width = 33;
