@@ -63,7 +63,7 @@ gboolean	 gcm_edid_parse				(GcmEdid		*edid,
 const gchar	*gcm_edid_get_monitor_name		(GcmEdid		*edid);
 const gchar	*gcm_edid_get_vendor_name		(GcmEdid		*edid);
 const gchar	*gcm_edid_get_serial_number		(GcmEdid		*edid);
-const gchar	*gcm_edid_get_ascii_string		(GcmEdid		*edid);
+const gchar	*gcm_edid_get_eisa_id		(GcmEdid		*edid);
 const gchar	*gcm_edid_get_pnp_id			(GcmEdid		*edid);
 guint		 gcm_edid_get_width			(GcmEdid		*edid);
 guint		 gcm_edid_get_height			(GcmEdid		*edid);
