@@ -56,6 +56,7 @@ static GcmColorimeter *colorimeter = NULL;
 static gboolean setting_up_device = FALSE;
 static GtkWidget *info_bar_loading = NULL;
 static GtkWidget *info_bar_vcgt = NULL;
+static GtkWidget *info_bar_profiles = NULL;
 static GtkWidget *cie_widget = NULL;
 static GtkWidget *trc_widget = NULL;
 static GConfClient *gconf_client = NULL;
@@ -94,6 +95,8 @@ typedef enum {
 
 static void gcm_prefs_devices_treeview_clicked_cb (GtkTreeSelection *selection, gpointer userdata);
 
+#define GCM_PREFS_TREEVIEW_WIDTH	350 /* px */
+
 /**
  * gcm_prefs_error_dialog:
  **/
@@ -684,22 +687,20 @@ gcm_prefs_ensure_argyllcms_installed (void)
 	window = GTK_WINDOW(gtk_builder_get_object (builder, "dialog_prefs"));
 	dialog = gtk_message_dialog_new (window, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE,
 					 /* TRANSLATORS: title, usually we can tell based on the EDID data or output name */
-					 _("Install missing calibration and profiling software?"));
+					 _("Install calibration and profiling software?"));
 
 	string = g_string_new ("");
 	/* TRANSLATORS: dialog message saying the argyllcms is not installed */
-	g_string_append_printf (string, "%s\n", _("Calibration and profiling software is not installed on this computer."));
+	g_string_append_printf (string, "%s\n", _("Calibration and profiling software is not installed."));
 	/* TRANSLATORS: dialog message saying the color targets are not installed */
-	g_string_append_printf (string, "%s\n\n", _("These tools are required to build color profiles for devices."));
-	/* TRANSLATORS: dialog message, asking if it's okay to install it */
-	g_string_append_printf (string, "%s", _("Do you want them to be automatically installed?"));
+	g_string_append_printf (string, "%s", _("These tools are required to build color profiles for devices."));
 
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), "%s", string->str);
 	gtk_window_set_icon_name (GTK_WINDOW (dialog), GCM_STOCK_ICON);
-	/* TRANSLATORS: button, install a package */
-	gtk_dialog_add_button (GTK_DIALOG (dialog), _("Install"), GTK_RESPONSE_YES);
 	/* TRANSLATORS: button, skip installing a package */
 	gtk_dialog_add_button (GTK_DIALOG (dialog), _("Do not install"), GTK_RESPONSE_CANCEL);
+	/* TRANSLATORS: button, install a package */
+	gtk_dialog_add_button (GTK_DIALOG (dialog), _("Install"), GTK_RESPONSE_YES);
 	response = gtk_dialog_run (GTK_DIALOG (dialog));
 	gtk_widget_destroy (dialog);
 
@@ -1001,17 +1002,19 @@ gcm_prefs_add_devices_columns (GtkTreeView *treeview)
 
 	/* image */
 	renderer = gtk_cell_renderer_pixbuf_new ();
-	g_object_set (renderer, "stock-size", GTK_ICON_SIZE_DIALOG, NULL);
+	g_object_set (renderer, "stock-size", GTK_ICON_SIZE_DND, NULL);
 	column = gtk_tree_view_column_new_with_attributes ("", renderer,
 							   "icon-name", GCM_DEVICES_COLUMN_ICON, NULL);
 	gtk_tree_view_append_column (treeview, column);
 
+	/* set minimum width */
+	gtk_widget_set_size_request (GTK_WIDGET (treeview), GCM_PREFS_TREEVIEW_WIDTH, -1);
+
 	/* column for text */
 	renderer = gtk_cell_renderer_text_new ();
 	g_object_set (renderer,
-		      "ellipsize", PANGO_ELLIPSIZE_END,
-		      "wrap-mode", PANGO_WRAP_WORD_CHAR,
-		      "width-chars", 40,
+		      "wrap-mode", PANGO_WRAP_WORD,
+		      "wrap-width", GCM_PREFS_TREEVIEW_WIDTH - 62,
 		      NULL);
 	column = gtk_tree_view_column_new_with_attributes ("", renderer,
 							   "markup", GCM_DEVICES_COLUMN_TITLE, NULL);
@@ -1037,12 +1040,14 @@ gcm_prefs_add_profiles_columns (GtkTreeView *treeview)
 							   "icon-name", GCM_PROFILES_COLUMN_ICON, NULL);
 	gtk_tree_view_append_column (treeview, column);
 
+	/* set minimum width */
+	gtk_widget_set_size_request (GTK_WIDGET (treeview), GCM_PREFS_TREEVIEW_WIDTH, -1);
+
 	/* column for text */
 	renderer = gtk_cell_renderer_text_new ();
 	g_object_set (renderer,
-		      "ellipsize", PANGO_ELLIPSIZE_END,
-		      "wrap-mode", PANGO_WRAP_WORD_CHAR,
-		      "width-chars", 50,
+		      "wrap-mode", PANGO_WRAP_WORD,
+		      "wrap-width", GCM_PREFS_TREEVIEW_WIDTH - 62,
 		      NULL);
 	column = gtk_tree_view_column_new_with_attributes ("", renderer,
 							   "markup", GCM_PROFILES_COLUMN_TITLE, NULL);
@@ -1083,7 +1088,7 @@ gcm_prefs_set_calibrate_button_sensitivity (void)
 		connected = gcm_device_get_connected (current_device);
 		if (!connected) {
 			/* TRANSLATORS: this is when the button is insensitive */
-			tooltip = _("Cannot calibrate: The device is not connected");
+			tooltip = _("Cannot calibrate: The display device is not connected");
 			goto out;
 		}
 
@@ -1248,6 +1253,7 @@ gcm_prefs_devices_treeview_clicked_cb (GtkTreeSelection *selection, gpointer use
 	const gchar *device_serial = NULL;
 	const gchar *device_model = NULL;
 	const gchar *device_manufacturer = NULL;
+	const gchar *eisa_id = NULL;
 
 	/* This will only work in single or browse selection mode! */
 	if (!gtk_tree_selection_get_selected (selection, &model, &iter)) {
@@ -1333,6 +1339,18 @@ gcm_prefs_devices_treeview_clicked_cb (GtkTreeSelection *selection, gpointer use
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "vbox_device_details"));
 	gtk_widget_show (widget);
 
+	/* get display specific properties */
+	if (kind == GCM_DEVICE_KIND_DISPLAY)
+		eisa_id = gcm_device_xrandr_get_eisa_id (GCM_DEVICE_XRANDR (current_device));
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "hbox_eisa"));
+	if (eisa_id != NULL) {
+		gtk_widget_show (widget);
+		widget = GTK_WIDGET (gtk_builder_get_object (builder, "label_eisa"));
+		gtk_label_set_label (GTK_LABEL (widget), eisa_id);
+	} else {
+		gtk_widget_hide (widget);
+	}
+
 	/* set adjustments */
 	setting_up_device = TRUE;
 	localgamma = gcm_device_get_gamma (current_device);
@@ -2180,6 +2198,7 @@ gcm_prefs_startup_phase2_idle_cb (gpointer user_data)
 	GtkWidget *widget;
 	GtkTreeSelection *selection;
 	GtkTreePath *path;
+	gboolean ret;
 
 	/* update list of profiles */
 	gcm_prefs_update_profile_list ();
@@ -2191,24 +2210,12 @@ gcm_prefs_startup_phase2_idle_cb (gpointer user_data)
 	gtk_tree_selection_select_path (selection, path);
 	gtk_tree_path_free (path);
 
-	return FALSE;
-}
+	/* do we show the shared-color-profiles-extra installer? */
+	egg_debug ("getting installed");
+	ret = gcm_utils_is_package_installed (GCM_PREFS_PACKAGE_NAME_COLOR_PROFILES_EXTRA);
+	gtk_widget_set_visible (info_bar_profiles, !ret);
 
-/**
- * gcm_prefs_colorspace_to_localised_string:
- **/
-static const gchar *
-gcm_prefs_colorspace_to_localised_string (GcmColorspace colorspace)
-{
-	if (colorspace == GCM_COLORSPACE_RGB) {
-		/* TRANSLATORS: this is the colorspace, e.g. red, green, blue */
-		return _("RGB");
-	}
-	if (colorspace == GCM_COLORSPACE_CMYK) {
-		/* TRANSLATORS: this is the colorspace, e.g. cyan, magenta, yellow, black */
-		return _("CMYK");
-	}
-	return NULL;
+	return FALSE;
 }
 
 /**
@@ -2220,19 +2227,14 @@ gcm_prefs_setup_space_combobox (GtkWidget *widget, GcmColorspace colorspace, con
 	GcmProfile *profile;
 	guint i;
 	const gchar *filename;
-	const gchar *description;
 	GcmColorspace colorspace_tmp;
 	gboolean has_profile = FALSE;
 	gboolean has_vcgt;
+	gboolean has_colorspace_description;
 	gchar *text = NULL;
-	const gchar *search = "RGB";
 	GPtrArray *profile_array = NULL;
 	GtkTreeIter iter;
 
-	/* search is a way to reduce to number of profiles */
-	if (colorspace == GCM_COLORSPACE_CMYK)
-		search = "CMYK";
-
 	/* get new list */
 	profile_array = gcm_profile_store_get_array (profile_store);
 
@@ -2241,13 +2243,13 @@ gcm_prefs_setup_space_combobox (GtkWidget *widget, GcmColorspace colorspace, con
 		profile = g_ptr_array_index (profile_array, i);
 
 		/* only for correct kind */
-		description = gcm_profile_get_description (profile);
 		has_vcgt = gcm_profile_get_has_vcgt (profile);
+		has_colorspace_description = gcm_profile_has_colorspace_description (profile);
 		colorspace_tmp = gcm_profile_get_colorspace (profile);
 		if (!has_vcgt &&
 		    colorspace == colorspace_tmp &&
 		    (colorspace == GCM_COLORSPACE_CMYK ||
-		     g_strstr_len (description, -1, search) != NULL)) {
+		     has_colorspace_description)) {
 			gcm_prefs_combobox_add_profile (widget, profile, GCM_PREFS_ENTRY_TYPE_PROFILE, &iter);
 
 			/* set active option */
@@ -2260,7 +2262,7 @@ gcm_prefs_setup_space_combobox (GtkWidget *widget, GcmColorspace colorspace, con
 	if (!has_profile) {
 		/* TRANSLATORS: this is when there are no profiles that can be used; the search term is either "RGB" or "CMYK" */
 		text = g_strdup_printf (_("No %s color spaces available"),
-					gcm_prefs_colorspace_to_localised_string (colorspace));
+					gcm_colorspace_to_localised_string (colorspace));
 		gtk_combo_box_append_text (GTK_COMBO_BOX(widget), text);
 		gtk_combo_box_set_active (GTK_COMBO_BOX (widget), 0);
 		gtk_widget_set_sensitive (widget, FALSE);
@@ -2389,7 +2391,7 @@ gcm_prefs_startup_phase1_idle_cb (gpointer user_data)
 
 	/* setup CMYK combobox */
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "combobox_space_cmyk"));
-	colorspace_cmyk = gconf_client_get_string (gconf_client, GCM_SETTINGS_COLORSPACE_CMYK, NULL);
+	colorspace_cmyk = gconf_client_get_string (gconf_client, GCM_SETTINGS_COLORSPACE_CMYK, &error);
 	if (colorspace_cmyk == NULL) {
 		egg_warning ("failed to get configuration value: %s", error->message);
 		g_clear_error (&error);
@@ -2402,7 +2404,7 @@ gcm_prefs_startup_phase1_idle_cb (gpointer user_data)
 	/* setup rendering lists */
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "combobox_rendering_display"));
 	gcm_prefs_set_combo_simple_text (widget);
-	intent_display = gconf_client_get_string (gconf_client, GCM_SETTINGS_RENDERING_INTENT_DISPLAY, NULL);
+	intent_display = gconf_client_get_string (gconf_client, GCM_SETTINGS_RENDERING_INTENT_DISPLAY, &error);
 	if (intent_display == NULL) {
 		egg_warning ("failed to get configuration value: %s", error->message);
 		g_clear_error (&error);
@@ -2413,7 +2415,7 @@ gcm_prefs_startup_phase1_idle_cb (gpointer user_data)
 
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "combobox_rendering_softproof"));
 	gcm_prefs_set_combo_simple_text (widget);
-	intent_softproof = gconf_client_get_string (gconf_client, GCM_SETTINGS_RENDERING_INTENT_SOFTPROOF, NULL);
+	intent_softproof = gconf_client_get_string (gconf_client, GCM_SETTINGS_RENDERING_INTENT_SOFTPROOF, &error);
 	if (intent_softproof == NULL) {
 		egg_warning ("failed to get configuration value: %s", error->message);
 		g_clear_error (&error);
@@ -2423,7 +2425,7 @@ gcm_prefs_startup_phase1_idle_cb (gpointer user_data)
 			  G_CALLBACK (gcm_prefs_renderer_combo_changed_cb), (gpointer) "softproof");
 
 	/* coldplug plugged in devices */
-	ret = gcm_client_add_connected (gcm_client, &error);
+	ret = gcm_client_add_connected (gcm_client, GCM_CLIENT_COLDPLUG_ALL, &error);
 	if (!ret) {
 		egg_warning ("failed to add connected devices: %s", error->message);
 		g_error_free (error);
@@ -2600,9 +2602,19 @@ gcm_prefs_client_notify_loading_cb (GcmClient *client, GParamSpec *pspec, gpoint
 static void
 gcm_prefs_info_bar_response_cb (GtkDialog *dialog, GtkResponseType response, gpointer user_data)
 {
+	GtkWindow *window;
+	gboolean ret;
+
 	if (response == GTK_RESPONSE_HELP) {
 		/* open the help file in the right place */
 		gcm_gnome_help ("faq-missing-vcgt");
+
+	} else if (response == GTK_RESPONSE_APPLY) {
+		/* install the extra profiles */
+		window = GTK_WINDOW(gtk_builder_get_object (builder, "dialog_prefs"));
+		ret = gcm_utils_install_package (GCM_PREFS_PACKAGE_NAME_COLOR_PROFILES_EXTRA, window);
+		if (ret)
+			gtk_widget_hide (info_bar_profiles);
 	}
 }
 
@@ -2686,8 +2698,7 @@ main (int argc, char **argv)
 	GtkTreeSelection *selection;
 	GtkWidget *info_bar_loading_label;
 	GtkWidget *info_bar_vcgt_label;
-	GtkSizeGroup *size_group = NULL;
-	GtkSizeGroup *size_group2 = NULL;
+	GtkWidget *info_bar_profiles_label;
 	GdkScreen *screen;
 
 	const GOptionEntry options[] = {
@@ -2886,60 +2897,6 @@ main (int argc, char **argv)
 	gtk_range_set_range (GTK_RANGE (widget), 0.1f, 1.0f);
 //	gtk_scale_add_mark (GTK_SCALE (widget), 1.0f, GTK_POS_TOP, "");
 
-	/* set alignment for left */
-	size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
-	widget = GTK_WIDGET (gtk_builder_get_object (builder, "hbox5"));
-	gtk_size_group_add_widget (size_group, widget);
-	widget = GTK_WIDGET (gtk_builder_get_object (builder, "hbox10"));
-	gtk_size_group_add_widget (size_group, widget);
-	widget = GTK_WIDGET (gtk_builder_get_object (builder, "hbox6"));
-	gtk_size_group_add_widget (size_group, widget);
-	widget = GTK_WIDGET (gtk_builder_get_object (builder, "hbox21"));
-	gtk_size_group_add_widget (size_group, widget);
-	widget = GTK_WIDGET (gtk_builder_get_object (builder, "hbox22"));
-	gtk_size_group_add_widget (size_group, widget);
-	widget = GTK_WIDGET (gtk_builder_get_object (builder, "hbox23"));
-	gtk_size_group_add_widget (size_group, widget);
-	widget = GTK_WIDGET (gtk_builder_get_object (builder, "hbox30"));
-	gtk_size_group_add_widget (size_group, widget);
-	widget = GTK_WIDGET (gtk_builder_get_object (builder, "hbox32"));
-	gtk_size_group_add_widget (size_group, widget);
-	widget = GTK_WIDGET (gtk_builder_get_object (builder, "hbox34"));
-	gtk_size_group_add_widget (size_group, widget);
-	widget = GTK_WIDGET (gtk_builder_get_object (builder, "hbox36"));
-	gtk_size_group_add_widget (size_group, widget);
-	widget = GTK_WIDGET (gtk_builder_get_object (builder, "hbox39"));
-	gtk_size_group_add_widget (size_group, widget);
-	widget = GTK_WIDGET (gtk_builder_get_object (builder, "hbox48"));
-	gtk_size_group_add_widget (size_group, widget);
-
-	/* set alignment for right */
-	size_group2 = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
-	widget = GTK_WIDGET (gtk_builder_get_object (builder, "hbox24"));
-	gtk_size_group_add_widget (size_group2, widget);
-	widget = GTK_WIDGET (gtk_builder_get_object (builder, "hbox25"));
-	gtk_size_group_add_widget (size_group2, widget);
-	widget = GTK_WIDGET (gtk_builder_get_object (builder, "hbox26"));
-	gtk_size_group_add_widget (size_group2, widget);
-	widget = GTK_WIDGET (gtk_builder_get_object (builder, "hbox11"));
-	gtk_size_group_add_widget (size_group2, widget);
-	widget = GTK_WIDGET (gtk_builder_get_object (builder, "hbox12"));
-	gtk_size_group_add_widget (size_group2, widget);
-	widget = GTK_WIDGET (gtk_builder_get_object (builder, "hbox18"));
-	gtk_size_group_add_widget (size_group2, widget);
-	widget = GTK_WIDGET (gtk_builder_get_object (builder, "hbox31"));
-	gtk_size_group_add_widget (size_group2, widget);
-	widget = GTK_WIDGET (gtk_builder_get_object (builder, "hbox33"));
-	gtk_size_group_add_widget (size_group2, widget);
-	widget = GTK_WIDGET (gtk_builder_get_object (builder, "hbox35"));
-	gtk_size_group_add_widget (size_group2, widget);
-	widget = GTK_WIDGET (gtk_builder_get_object (builder, "hbox37"));
-	gtk_size_group_add_widget (size_group2, widget);
-	widget = GTK_WIDGET (gtk_builder_get_object (builder, "hbox40"));
-	gtk_size_group_add_widget (size_group2, widget);
-	widget = GTK_WIDGET (gtk_builder_get_object (builder, "hbox49"));
-	gtk_size_group_add_widget (size_group2, widget);
-
 	/* use a device client array */
 	gcm_client = gcm_client_new ();
 	gcm_client_set_use_threads (gcm_client, TRUE);
@@ -2987,10 +2944,16 @@ main (int argc, char **argv)
 	info_bar_vcgt = gtk_info_bar_new ();
 	g_signal_connect (info_bar_vcgt, "response",
 			  G_CALLBACK (gcm_prefs_info_bar_response_cb), NULL);
+	info_bar_profiles = gtk_info_bar_new ();
+	g_signal_connect (info_bar_profiles, "response",
+			  G_CALLBACK (gcm_prefs_info_bar_response_cb), NULL);
 
 	/* TRANSLATORS: button for more details about the vcgt failure */
 	gtk_info_bar_add_button (GTK_INFO_BAR(info_bar_vcgt), _("More Information"), GTK_RESPONSE_HELP);
 
+	/* TRANSLATORS: button to install extra profiles */
+	gtk_info_bar_add_button (GTK_INFO_BAR(info_bar_profiles), _("Install now"), GTK_RESPONSE_APPLY);
+
 	/* TRANSLATORS: this is displayed while the devices are being probed */
 	info_bar_loading_label = gtk_label_new (_("Loading list of devices…"));
 	gtk_info_bar_set_message_type (GTK_INFO_BAR(info_bar_loading), GTK_MESSAGE_INFO);
@@ -3000,11 +2963,20 @@ main (int argc, char **argv)
 
 	/* TRANSLATORS: this is displayed when the profile is crap */
 	info_bar_vcgt_label = gtk_label_new (_("This profile does not have the information required for whole-screen color correction."));
+	gtk_label_set_line_wrap (GTK_LABEL (info_bar_vcgt_label), TRUE);
 	gtk_info_bar_set_message_type (GTK_INFO_BAR(info_bar_vcgt), GTK_MESSAGE_INFO);
 	widget = gtk_info_bar_get_content_area (GTK_INFO_BAR(info_bar_vcgt));
 	gtk_container_add (GTK_CONTAINER(widget), info_bar_vcgt_label);
 	gtk_widget_show (info_bar_vcgt_label);
 
+	/* TRANSLATORS: this is displayed when the profile is crap */
+	info_bar_profiles_label = gtk_label_new (_("More color profiles could be automatically installed."));
+	gtk_label_set_line_wrap (GTK_LABEL (info_bar_profiles_label), TRUE);
+	gtk_info_bar_set_message_type (GTK_INFO_BAR(info_bar_profiles), GTK_MESSAGE_INFO);
+	widget = gtk_info_bar_get_content_area (GTK_INFO_BAR(info_bar_profiles));
+	gtk_container_add (GTK_CONTAINER(widget), info_bar_profiles_label);
+	gtk_widget_show (info_bar_profiles_label);
+
 	/* add infobar to devices pane */
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "vbox_devices"));
 	gtk_box_pack_start (GTK_BOX(widget), info_bar_loading, FALSE, FALSE, 0);
@@ -3013,8 +2985,11 @@ main (int argc, char **argv)
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "vbox_sections"));
 	gtk_box_pack_start (GTK_BOX(widget), info_bar_vcgt, FALSE, FALSE, 0);
 
+	/* add infobar to defaults pane */
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "vbox3"));
+	gtk_box_pack_start (GTK_BOX(widget), info_bar_profiles, TRUE, FALSE, 0);
+
 	/* show main UI */
-	gtk_window_set_default_size (GTK_WINDOW(main_window), 1000, 450);
 	gtk_widget_show (main_window);
 
 	/* connect up sliders */
@@ -3050,10 +3025,6 @@ main (int argc, char **argv)
 out:
 	g_object_unref (unique_app);
 	g_main_loop_unref (loop);
-	if (size_group != NULL)
-		g_object_unref (size_group);
-	if (size_group2 != NULL)
-		g_object_unref (size_group2);
 	if (current_device != NULL)
 		g_object_unref (current_device);
 	if (colorimeter != NULL)
