@@ -55,7 +55,6 @@ struct _GcmProfilePrivate
 	GcmColorspace		 colorspace;
 	guint			 size;
 	gboolean		 has_vcgt;
-	gboolean		 can_delete;
 	gchar			*description;
 	gchar			*filename;
 	gchar			*copyright;
@@ -82,7 +81,6 @@ enum {
 	PROP_COLORSPACE,
 	PROP_SIZE,
 	PROP_HAS_VCGT,
-	PROP_CAN_DELETE,
 	PROP_WHITE,
 	PROP_BLACK,
 	PROP_RED,
@@ -144,6 +142,28 @@ gcm_profile_get_filename (GcmProfile *profile)
 	return profile->priv->filename;
 }
 
+/**
+ * gcm_profile_has_colorspace_description:
+ *
+ * Return value: if the description mentions the profile colorspace explicity,
+ * e.g. "Adobe RGB" for %GCM_COLORSPACE_RGB.
+ **/
+gboolean
+gcm_profile_has_colorspace_description (GcmProfile *profile)
+{
+	GcmProfilePrivate *priv = profile->priv;
+	g_return_val_if_fail (GCM_IS_PROFILE (profile), FALSE);
+
+	/* for each profile type */
+	if (priv->colorspace == GCM_COLORSPACE_RGB)
+		return (g_strstr_len (priv->description, -1, "RGB") != NULL);
+	if (priv->colorspace == GCM_COLORSPACE_CMYK)
+		return (g_strstr_len (priv->description, -1, "CMYK") != NULL);
+
+	/* nothing */
+	return FALSE;
+}
+
 /**
  * gcm_profile_set_filename:
  **/
@@ -372,16 +392,6 @@ gcm_profile_set_has_vcgt (GcmProfile *profile, gboolean has_vcgt)
 	g_object_notify (G_OBJECT (profile), "has_vcgt");
 }
 
-/**
- * gcm_profile_get_can_delete:
- **/
-gboolean
-gcm_profile_get_can_delete (GcmProfile *profile)
-{
-	g_return_val_if_fail (GCM_IS_PROFILE (profile), FALSE);
-	return profile->priv->can_delete;
-}
-
 /**
  * gcm_profile_parse_data:
  **/
@@ -414,22 +424,14 @@ gboolean
 gcm_profile_parse (GcmProfile *profile, GFile *file, GError **error)
 {
 	gchar *data = NULL;
-	gboolean ret = FALSE;
+	gboolean ret;
 	gsize length;
 	gchar *filename = NULL;
 	GError *error_local = NULL;
-	GFileInfo *info;
 
 	g_return_val_if_fail (GCM_IS_PROFILE (profile), FALSE);
 	g_return_val_if_fail (file != NULL, FALSE);
 
-	/* find out if the user could delete this profile */
-	info = g_file_query_info (file, G_FILE_ATTRIBUTE_ACCESS_CAN_DELETE,
-				  G_FILE_QUERY_INFO_NONE, NULL, error);
-	if (info == NULL)
-		goto out;
-	profile->priv->can_delete = g_file_info_get_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_DELETE);
-
 	/* load files */
 	ret = g_file_load_contents (file, NULL, &data, &length, NULL, &error_local);
 	if (!ret) {
@@ -447,8 +449,6 @@ gcm_profile_parse (GcmProfile *profile, GFile *file, GError **error)
 	filename = g_file_get_path (file);
 	gcm_profile_set_filename (profile, filename);
 out:
-	if (info != NULL)
-		g_object_unref (info);
 	g_free (filename);
 	g_free (data);
 	return ret;
@@ -583,9 +583,6 @@ gcm_profile_get_property (GObject *object, guint prop_id, GValue *value, GParamS
 	case PROP_HAS_VCGT:
 		g_value_set_boolean (value, priv->has_vcgt);
 		break;
-	case PROP_CAN_DELETE:
-		g_value_set_boolean (value, priv->can_delete);
-		break;
 	case PROP_WHITE:
 		g_value_set_object (value, priv->white);
 		break;
@@ -760,14 +757,6 @@ gcm_profile_class_init (GcmProfileClass *klass)
 				      G_PARAM_READWRITE);
 	g_object_class_install_property (object_class, PROP_HAS_VCGT, pspec);
 
-	/**
-	 * GcmProfile:can-delete:
-	 */
-	pspec = g_param_spec_boolean ("can-delete", NULL, NULL,
-				      FALSE,
-				      G_PARAM_READABLE);
-	g_object_class_install_property (object_class, PROP_CAN_DELETE, pspec);
-
 	/**
 	 * GcmProfile:white:
 	 */
@@ -818,7 +807,6 @@ static void
 gcm_profile_init (GcmProfile *profile)
 {
 	profile->priv = GCM_PROFILE_GET_PRIVATE (profile);
-	profile->priv->can_delete = FALSE;
 	profile->priv->monitor = NULL;
 	profile->priv->kind = GCM_PROFILE_KIND_UNKNOWN;
 	profile->priv->colorspace = GCM_COLORSPACE_UNKNOWN;
@@ -908,7 +896,7 @@ gcm_profile_test_parse_file (EggTest *test, const guint8 *datafile, GcmProfileTe
 	const gchar *model;
 	const gchar *datetime;
 	const gchar *description;
-	const gchar *ascii_string;
+	const gchar *eisa_id;
 	const gchar *pnp_id;
 	guint width;
 	guint kind;
