@@ -83,7 +83,6 @@ gboolean	 gcm_profile_parse_data			(GcmProfile	*profile,
 gboolean	 gcm_profile_save			(GcmProfile	*profile,
 							 const gchar	*filename,
 							 GError		**error);
-gboolean	 gcm_profile_get_can_delete		(GcmProfile	*profile);
 GcmClut		*gcm_profile_generate_vcgt		(GcmProfile	*profile,
 							 guint		 size);
 GcmClut		*gcm_profile_generate_curve		(GcmProfile	*profile,
@@ -118,6 +117,7 @@ void		 gcm_profile_set_colorspace		(GcmProfile	*profile,
 gboolean	 gcm_profile_get_has_vcgt		(GcmProfile	*profile);
 void		 gcm_profile_set_has_vcgt		(GcmProfile	*profile,
 							 gboolean	 has_vcgt);
+gboolean	 gcm_profile_has_colorspace_description	(GcmProfile	*profile);
 
 G_END_DECLS
 
