@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2009 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2009-2010 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -180,6 +180,54 @@ out:
 	return ret;
 }
 
+/**
+ * gcm_utils_is_package_installed:
+ **/
+gboolean
+gcm_utils_is_package_installed (const gchar *package_name)
+{
+	DBusGConnection *connection;
+	DBusGProxy *proxy;
+	GError *error = NULL;
+	gboolean ret;
+	gboolean installed = TRUE;
+
+	g_return_val_if_fail (package_name != NULL, FALSE);
+
+#ifndef GCM_USE_PACKAGEKIT
+	egg_warning ("cannot query %s: this package was not compiled with --enable-packagekit", package_name);
+	return TRUE;
+#endif
+
+	/* get a session bus connection */
+	connection = dbus_g_bus_get (DBUS_BUS_SESSION, NULL);
+
+	/* connect to PackageKit */
+	proxy = dbus_g_proxy_new_for_name (connection,
+					   "org.freedesktop.PackageKit",
+					   "/org/freedesktop/PackageKit",
+					   "org.freedesktop.PackageKit.Query");
+
+	/* set timeout */
+	dbus_g_proxy_set_default_timeout (proxy, G_MAXINT);
+
+	/* execute sync method */
+	ret = dbus_g_proxy_call (proxy, "IsInstalled", &error,
+				 G_TYPE_STRING, package_name,
+				 G_TYPE_STRING, "timeout=5",
+				 G_TYPE_INVALID,
+				 G_TYPE_BOOLEAN, &installed,
+				 G_TYPE_INVALID);
+	if (!ret) {
+		egg_warning ("failed to get installed status: %s", error->message);
+		g_error_free (error);
+		goto out;
+	}
+out:
+	g_object_unref (proxy);
+	return installed;
+}
+
 /**
  * gcm_utils_output_is_lcd_internal:
  * @output_name: the output name
@@ -478,28 +526,28 @@ gcm_utils_get_default_config_location (void)
 }
 
 /**
- * gcm_utils_device_type_to_profile_type:
+ * gcm_utils_device_kind_to_profile_kind:
  **/
-GcmProfileTypeEnum
-gcm_utils_device_type_to_profile_type (GcmDeviceTypeEnum type)
+GcmProfileKind
+gcm_utils_device_kind_to_profile_kind (GcmDeviceKind kind)
 {
-	GcmProfileTypeEnum profile_type;
-	switch (type) {
-	case GCM_DEVICE_TYPE_ENUM_DISPLAY:
-		profile_type = GCM_PROFILE_TYPE_ENUM_DISPLAY_DEVICE;
+	GcmProfileKind profile_kind;
+	switch (kind) {
+	case GCM_DEVICE_KIND_DISPLAY:
+		profile_kind = GCM_PROFILE_KIND_DISPLAY_DEVICE;
 		break;
-	case GCM_DEVICE_TYPE_ENUM_CAMERA:
-	case GCM_DEVICE_TYPE_ENUM_SCANNER:
-		profile_type = GCM_PROFILE_TYPE_ENUM_INPUT_DEVICE;
+	case GCM_DEVICE_KIND_CAMERA:
+	case GCM_DEVICE_KIND_SCANNER:
+		profile_kind = GCM_PROFILE_KIND_INPUT_DEVICE;
 		break;
-	case GCM_DEVICE_TYPE_ENUM_PRINTER:
-		profile_type = GCM_PROFILE_TYPE_ENUM_OUTPUT_DEVICE;
+	case GCM_DEVICE_KIND_PRINTER:
+		profile_kind = GCM_PROFILE_KIND_OUTPUT_DEVICE;
 		break;
 	default:
-		egg_warning ("unknown type: %i", type);
-		profile_type = GCM_PROFILE_TYPE_ENUM_UNKNOWN;
+		egg_warning ("unknown kind: %i", kind);
+		profile_kind = GCM_PROFILE_KIND_UNKNOWN;
 	}
-	return profile_type;
+	return profile_kind;
 }
 
 /**
@@ -517,24 +565,24 @@ gcm_utils_format_date_time (const struct tm *created)
 }
 
 /**
- * gcm_intent_enum_to_localized_text:
+ * gcm_intent_to_localized_text:
  **/
 const gchar *
-gcm_intent_enum_to_localized_text (GcmIntentEnum intent)
+gcm_intent_to_localized_text (GcmIntent intent)
 {
-	if (intent == GCM_INTENT_ENUM_PERCEPTUAL) {
+	if (intent == GCM_INTENT_PERCEPTUAL) {
 		/* TRANSLATORS: rendering intent: you probably want to google this */
 		return _("Perceptual");
 	}
-	if (intent == GCM_INTENT_ENUM_RELATIVE_COLORMETRIC) {
+	if (intent == GCM_INTENT_RELATIVE_COLORMETRIC) {
 		/* TRANSLATORS: rendering intent: you probably want to google this */
 		return _("Relative colormetric");
 	}
-	if (intent == GCM_INTENT_ENUM_SATURATION) {
+	if (intent == GCM_INTENT_SATURATION) {
 		/* TRANSLATORS: rendering intent: you probably want to google this */
 		return _("Saturation");
 	}
-	if (intent == GCM_INTENT_ENUM_ABSOLUTE_COLORMETRIC) {
+	if (intent == GCM_INTENT_ABSOLUTE_COLORMETRIC) {
 		/* TRANSLATORS: rendering intent: you probably want to google this */
 		return _("Absolute colormetric");
 	}
@@ -555,8 +603,8 @@ gcm_utils_test (EggTest *test)
 	GPtrArray *array;
 	gchar *text;
 	gchar *filename;
-	GcmProfileTypeEnum profile_type;
-	GcmDeviceTypeEnum device_type;
+	GcmProfileKind profile_kind;
+	GcmDeviceKind device_kind;
 	GFile *file;
 	GFile *dest;
 
@@ -656,14 +704,14 @@ gcm_utils_test (EggTest *test)
 	g_free (filename);
 
 	/************************************************************/
-	egg_test_title (test, "convert valid device type to profile type");
-	profile_type = gcm_utils_device_type_to_profile_type (GCM_DEVICE_TYPE_ENUM_SCANNER);
-	egg_test_assert (test, (profile_type == GCM_PROFILE_TYPE_ENUM_INPUT_DEVICE));
+	egg_test_title (test, "convert valid device kind to profile kind");
+	profile_kind = gcm_utils_device_kind_to_profile_kind (GCM_DEVICE_KIND_SCANNER);
+	egg_test_assert (test, (profile_kind == GCM_PROFILE_KIND_INPUT_DEVICE));
 
 	/************************************************************/
-	egg_test_title (test, "convert invalid device type to profile type");
-	profile_type = gcm_utils_device_type_to_profile_type (GCM_DEVICE_TYPE_ENUM_UNKNOWN);
-	egg_test_assert (test, (profile_type == GCM_PROFILE_TYPE_ENUM_UNKNOWN));
+	egg_test_title (test, "convert invalid device kind to profile kind");
+	profile_kind = gcm_utils_device_kind_to_profile_kind (GCM_DEVICE_KIND_UNKNOWN);
+	egg_test_assert (test, (profile_kind == GCM_PROFILE_KIND_UNKNOWN));
 
 	egg_test_end (test);
 }
